/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import kumbang.modeller.util.Constants;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.OpenPerspectiveAction;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class KumbangModellerPlugin extends AbstractUIPlugin {

	//The shared instance.
	private static KumbangModellerPlugin plugin;

	public final static String PLUGIN_ID = "kumbang.modeller";
	public static final String TYPE_VIEW_ID = "kumbang.modeller.TypeView";
	public static final String COMPONENT_VIEW_ID = "kumbang.modeller.ComponentView";
	public static final String FEATURE_VIEW_ID = "kumbang.modeller.FeatureView";
	public static final String PERSPECTIVE_ID = "kumbang.modeller.Perspective";

	private ColorRegistry colorRegistry;

	private IPropertyChangeListener listener;

	/**
	 * The constructor.
	 */
	public KumbangModellerPlugin() {
		plugin = this;
	}

	
	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		this.colorRegistry = JFaceResources.getColorRegistry();
		initColors();
		listener = new IPropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent event) {
				// TODO Auto-generated method stub
				
			}};
		
		getPreferenceStore().addPropertyChangeListener(listener);
				
	}


	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
		getPreferenceStore().removePropertyChangeListener(listener);
	}

	/**
	 * Returns the shared instance.
	 */
	public static KumbangModellerPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	public ColorRegistry getColorRegistry() {
		return  colorRegistry;
	}
	
	private void initColors() {	
		colorRegistry.put(Constants.FEATURE_TYPE_COLOR_STRING, new RGB (10, 75, 40));
		colorRegistry.put(Constants.COMPONENT_TYPE_COLOR_STRING, new RGB(10, 30, 85));	
		colorRegistry.put(Constants.INTERFACE_TYPE_COLOR_STRING, new RGB (65, 65, 10));
		colorRegistry.put(Constants.ATTRIBUTE_VALUE_COLOR_STRING, new RGB (85, 10, 10));
		
		colorRegistry.put(Constants.SUBFEATURE_DEFINITION_COLOR_STRING, new RGB (15, 125, 95));
		colorRegistry.put(Constants.PART_DEFINITION_COLOR_STRING,  new RGB (50, 75, 120));
		colorRegistry.put(Constants.INTERFACE_DEFINITION_COLOR_STRING, new RGB(100, 100, 30));
		colorRegistry.put(Constants.ATTRIBUTE_DEFINITION_COLOR_STRING, new RGB(120, 50, 50));
		
		colorRegistry.put(Constants.COLOR_WHITE, new RGB(255, 255, 255));		
	}

	/**
	 * Returns the plugin path as a File object
	 * @return plugin path
	 * @throws IOException
	 */





}
