/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.ui.views.properties.IPropertySource;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Name;
import kumbang.modeller.ui.properties.AttributeDefinitionPropertySource;
import kumbang.modeller.ui.properties.AttributeValueTypePropertySource;

public class AttributeValueTypeController extends BasicController {

	private AttributeValueType type;
	private AttributeDefinition definition;
	private ComposableType superType;


	public AttributeValueTypeController(ModelController modelController,AttributeValueType type) {
		super(modelController);
		this.type = type;
	}

	
	public void setDefinition(AttributeDefinition definition) {
		this.definition = definition;
	}
	
	public void setSuperType(ComposableType type) {
		this.superType = type;
	}
	
	public Object[] getChildren() {
		return new Object[0];
	}

	public boolean hasChildren() {
		return false;
	}

	public String getName() {
		return type.getName();
	}

	public int getType() {
		return ATTRIBUTE_VALUE_TYPE;
	}

	public String getDisplayText() {
		return type.getName();
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new AttributeValueTypePropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public String getImageString() {
		return BasicController.ATTRIBUTE_VALUE_TYPE_IMAGE_STRING;
	}

	public void delete() {
		if (definition == null)
			modelController.removeAttributeValue(type.getName(), false);
		else {
			superType.removeAttribute(definition);
			modified();
		}	
	}
	
	public String getBackgroundString() {	
		return ATTRIBUTE_VALUE_COLOR_STRING;
	}

	public Name[] getValues() {
		Iterator it = type.getValues();
		LinkedList<Name> list = new LinkedList<Name>();
		while(it.hasNext()) {
			list.add((Name) it.next());
		}
		Name[] names = new Name[list.size()];
		for (int i = 0; i < list.size(); i++)
			names[i] = ((Name)list.get(i));
		return names;
	}


	public void setValues(HashMap<String, Name> names) throws PropertyAlreadyDefinedException {
		type.setValues(names.values().iterator());
		modified();
		
	}
	
	public void setName(String name) {
		modelController.updateTypeName(type, name);
		
		
	}

	public String getDisplayName() {
		if (type.hasDisplayName())
			return type.getDisplayName();
		return null;
	}


	public void setDisplayName(String text) {
		type.setDisplayName(text);
		modified();
		
	}


	



}
