/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.constraint.Expression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.exception.OnesOwnPartException;
import kumbang.core.model.exception.OnesOwnSupertypeException;
import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;

public abstract class ComposableTypeController  extends BasicController {

	protected ComposableType type;

	public ComposableTypeController(ModelController modelController, ComposableType type) {
		super(modelController);
		this.type = type;
	
	}
	
	public ComposableType getComposableType() {
		return type;
	}
	
	public void addPartDefinition(Name partName, String[] types, int min, int max, int similarity) throws OnesOwnPartException, PropertyAlreadyDefinedException {
		List typeList = new ArrayList(types.length);
		for (int i = 0; i < types.length; i++)
			typeList.add(modelController.getComposableType(types[i]));
		PartDefinition part = new PartDefinition(partName, typeList, min, max, similarity);
		type.addPart(part, true);
		modified();
		
	}
	
	public void removePartDefinition(String partName, boolean isUndo) {	
		type.removePart(type.getPart(partName));
		modified(isUndo);
	}
	
	public String getName() {
		return type.getName();
	}
	

	public String getDisplayText() {
		return type.getName();
	}

	public void addAttributeDefinition(Name name, String typeName) throws Exception {
		type.addAttribute(new AttributeDefinition(name, modelController.getAttributeType(typeName), false));	
		modified();
	}



	public void removeAttributeDefinition(String name) {
		// TODO Auto-generated method stub
		
	}

	public void addDefinition(PartDefinition newDefinition) throws  PropertyAlreadyDefinedException, OnesOwnPartException {
		type.addPart(newDefinition, true);
		modified();
		
	}

	public void addConstraint(BooleanExpression constraint) {
		
		type.addConstraint(constraint);
		modified();
	}

	public void removeConstraint(BooleanExpression expression) {
		type.removeConstraint(expression);
		modified();
		
	}
	
	public String getDisplayName() {
		return type.getDisplayName();
		
	}
	
	public void setDisplayName(String text) {
		type.setDisplayName(text);
		modified();
	}

	public void setName(String newName) {
		modelController.updateTypeName(type, newName);
	}

	public String[] getParts() {
		Object[] parts = type.getPartsArray();
		String[] defs = new String[parts.length];
		
		for (int i = 0; i < defs.length; i++) {
			PartDefinition part = (PartDefinition) parts[i];
			// Perhaps a check to find out if the part has any interfaces?
			defs[i] = part.getName();
		}
		
		return defs;
	}

	public String[] getAttributes() {
		Iterator<AttributeDefinition> attributes = type.getAttributes();   
		
		Vector<String> names = new Vector<String>();
		
		while (attributes.hasNext())
			names.add(attributes.next().getName());
		
		return names.toArray(new String[names.size()]);
		
//		Object[] attributes = type.getAttributesArray();
//		String[] defs = new String[attributes.length];
//		
//		for (int i = 0; i < defs.length; i++) {
//			AttributeDefinition part = (AttributeDefinition) attributes[i];
//			
//			defs[i] = part.getName();
//		}
//		
//		return defs;
	}

	public PartDefinition getPartDefinition(String name) {
		return type.getPart(name);
	}

	public AttributeDefinition getAttribute(String name) {
		return type.getAttributeDefinition(name);	
		}

	@SuppressWarnings("unchecked")
	public Iterator<ComposableType> getSupertypes() {
		return ((ComposableType)type).getSupertypes();
	
	}
	
	public boolean isAbstract() {
		return type.isAbstract();
	}
	
	
	public void setAbstract(boolean selection) {
		type.setAbstract(selection);
		modified();
		
	}
	public void setSupertypes(Collection<ComposableType> name) throws OnesOwnSupertypeException {
		type.setSupertypes(new ArrayList<ComposableType>(name));
		modified();
	}

//	public PartDefinition getPartDefinition(StructureReference ref) {
//		PartDefinition currentPart = null;
//		for (int i = 0; i < ref.getDepth(); i++) {
//			String partName = ref.getName(i);
//			if (currentPart == null)
//				currentPart = getPartDefinition(partName);
//			else {
//				for (Iterator<ComposableType> it = currentPart.getPossibleTypes(); it.hasNext();) {
//					ComposableType c = it.next();
//					if (c.getPart(partName) != null)
//						currentPart = c.getPart(partName);
//					
//				}
//			}
//			
//		}
//		return currentPart;
//	}



}

