/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import org.eclipse.ui.views.properties.IPropertySource;

import kumbang.core.model.type.Connection;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.ui.properties.AttributeDefinitionPropertySource;
import kumbang.modeller.ui.properties.ConnectionPropertySource;

public class ConnectionController extends BasicController {

	private Connection connection;
	private ComponentTypeController parent;

	
	
	public ConnectionController(ModelController modelController, Connection connection) {
		super(modelController);
		this.connection = connection;
	}

	public String getName() {
		return getSourceString() + " = " + getTargetString();
	}

	public Object[] getChildren() {
		return new Object[0];
	}

	public boolean hasChildren() {
		return false;
	}

	public int getType() {
		return CONNECTION;
	}

	public String getDisplayText() {
		return getName();
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new ConnectionPropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public String getImageString() {
		return BasicController.CONNECTION_IMAGE_STRING;
	}

	public void delete() {
		parent.removeConnection(this);
		
		
	}
	
	public String getBackgroundString() {
		return CONNECTION_COLOR_STRING;
	}

	public String getDisplayName() {
		return null;
	}

	
	public PartDefinition getSourcePart() {
		return connection.getSourcePart();
	}
	
	public InterfaceDefinition getSourceInterface() {
		return connection.getSourceInterface();
	}
	
	public PartDefinition getTargetPart() {
		return connection.getTargetPart();
	}
	
	public InterfaceDefinition getTargetInterface() {
		return connection.getTargetInterface();
	}

	public Connection getConnection() {
		return connection;
	}

	public void setParent(ComponentTypeController controller) {
		this.parent = controller;
		
	}
	
	public ComponentTypeController getParent() {
		return parent;
	}

	public String getSourceString() {
		String s ="";
		
		if (connection.getSourcePart() != null)
			 s += connection.getSourcePart().getName() + "->";
		s += connection.getSourceInterface().getName();
		return s;
	}
	
	public String getTargetString() {
		String s ="";
		
		if (connection.getTargetPart() != null)
			 s += connection.getTargetPart().getName() + "->";
		s += connection.getTargetInterface().getName();
		return s;
	}
	
}
