/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;


import java.util.Iterator;

import kumbang.core.model.constraint.AttributeExpression;
import kumbang.core.model.constraint.BinaryExpression;
import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.constraint.CardinalityExpression;
import kumbang.core.model.constraint.ComparisonExpression;
import kumbang.core.model.constraint.ConjunctiveExpression;
import kumbang.core.model.constraint.ConnectedExpression;
import kumbang.core.model.constraint.EquivalenceExpression;
import kumbang.core.model.constraint.Expression;
import kumbang.core.model.constraint.HasPartOfTypeExpression;
import kumbang.core.model.constraint.ImplicationExpression;
import kumbang.core.model.constraint.InstanceOfExpression;
import kumbang.core.model.constraint.QuantifiedExpression;
import kumbang.core.model.constraint.ReferenceExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.modeller.ui.properties.ConstraintPropertySource;

import org.eclipse.ui.views.properties.IPropertySource;

public class ConstraintController extends BasicController {

	private BooleanExpression expression;
	private ComposableTypeController parentController;


	public ConstraintController(ModelController modelController, BooleanExpression expression) {
		super(modelController);
		this.expression = expression;
	}

	public void setParent(ComposableTypeController parent) {
		this.parentController = parent;
	}
	
	public int getType() {
		return CONSTRAINT;
	}

	public String getName() {
		return expression.getModelString(parentController instanceof FeatureTypeController ? true : false);
	}

	public String getDisplayText() {
		return getName();
	}

	public Object[] getChildren() {
		return new Object[0];
	}

	public boolean hasChildren() {
		return false;
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new ConstraintPropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public String getImageString() {
		return BasicController.CONSTRAINT_IMAGE_STRING;
	}

	public void delete() {
		parentController.removeConstraint(expression);
		
		
	}

	public String getBackgroundString() {
		return CONSTRAINT_COLOR_STRING;
	}

	@Override
	public String getDisplayName() {
		// TODO Auto-generated method stub
		return null;
	}

	public BooleanExpression getExpression() {
		return expression;
	}

	public static boolean isImplementation(Expression expression) {
		if (expression instanceof ComparisonExpression) {
			return isImplementation(((ComparisonExpression)expression).getLeft()) || isImplementation(((ComparisonExpression)expression).getRight());
		}
		
		if (expression instanceof ConjunctiveExpression) {
			for (Iterator<Expression> it =  ((ConjunctiveExpression)expression).getPrimitives(); it.hasNext();) {
				if (isImplementation(it.next()))
					return true;
			}
		}
		
		if (expression instanceof EquivalenceExpression) {
			return isImplementation(((EquivalenceExpression)expression).getLeft()) || isImplementation(((EquivalenceExpression)expression).getRight());
		}	
		
		if (expression instanceof HasPartOfTypeExpression)
			return ((HasPartOfTypeExpression)expression).getReference().isImplementation();
		
		if (expression instanceof ImplicationExpression) {
			return isImplementation(((ImplicationExpression)expression).getHead()) || isImplementation(((ImplicationExpression)expression).getTail());
		}
		
		if (expression instanceof ConnectedExpression)
			return true; // As this method is only called for features, no need to check anything elses
		
		if (expression instanceof ReferenceExpression)
			return ((ReferenceExpression)expression).getReference().isImplementation();
		
		if (expression instanceof QuantifiedExpression)
			return isImplementation(((QuantifiedExpression)expression).getExpression());
		
		if (expression instanceof BinaryExpression) {
			return isImplementation(((BinaryExpression)expression).getLeft()) || isImplementation(((BinaryExpression)expression).getRight());
		}
		
		if (expression instanceof AttributeExpression) {
			StructureReference ref = ((AttributeExpression)expression).getStructureReference();
			if (ref != null)
				return ref.isImplementation();
		}
		if (expression instanceof CardinalityExpression)
			return ((CardinalityExpression)expression).getReference().isImplementation();
		
		
//		if (expression instanceof InstanceOfExpression) {
//			return ((InstanceOfExpression)expression).getReference().isImplementation();
//		}
		
		return false;
	}

	public ComposableTypeController getParentController() {
		return parentController;
	}


}
