/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.HashMap;

import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Connection;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.PartDefinition;

public class ControllerFactory {


	private HashMap types = new HashMap();
	
	private ModelController modelController;
	
	ControllerFactory( ModelController modelController) {
		this.modelController = modelController;
	}
	
	public BasicController createController(FeatureType type) {
		BasicController stored = (BasicController) types.get(type);
		if (stored == null) {		
			stored = new FeatureTypeController(modelController, type);
			types.put(type, stored);
		}
		return stored;
	
	}
	
	public BasicController createController(ComponentType type) {
		BasicController stored = (BasicController) types.get(type);
		if (stored != null)
			return stored;
		stored = new ComponentTypeController(modelController, type);
		types.put(type, stored);
		return stored;
	}
	
	public BasicController createController(InterfaceType type) {
		BasicController stored = (BasicController) types.get(type);
		if (stored != null)
			return stored;
		stored = new InterfaceTypeController(modelController, type);
		types.put(type, stored);
		return stored;
	}
	
	public BasicController createController(AttributeValueType type) {
		BasicController stored = (BasicController) types.get(type);
		if (stored != null)
			return stored;
		stored = new AttributeValueTypeController(modelController, type);
		types.put(type, stored);
		return stored;
	}
	
	
	public BasicController createController(PartDefinition definition) {
		BasicController stored = (BasicController) types.get(definition);
		if (stored == null) {		
			stored = new PartDefinitionController(modelController, definition);
			types.put(definition, stored);
		}
		return stored;
	
	}
	
	public BasicController createController(InterfaceDefinition definition) {
		BasicController stored = (BasicController) types.get(definition);
		if (stored == null) {		
			stored = new InterfaceDefinitionController(modelController, definition);
			types.put(definition, stored);
		}
		return stored;
	}
	
	public BasicController createController(AttributeDefinition definition) {
		BasicController stored = (BasicController) types.get(definition);
		if (stored == null) {		
			stored = new AttributeDefinitionController(modelController, definition);
			types.put(definition, stored);
		}
		return stored;
	}
	
	public BasicController createController(BooleanExpression expression) {
		BasicController stored = (BasicController) types.get(expression);
		if (stored == null) {		
			stored = new ConstraintController(modelController, expression);
			types.put(expression, stored);
		}
		return stored;
	}
	
	public BasicController createController(Connection connection) {
		BasicController stored = (BasicController) types.get(connection);
		if (stored == null) {		
			stored = new ConnectionController(modelController, connection);
			types.put(connection, stored);
		}
		return stored;
	}
	
	
	public BasicController createController(Object arg) {
		
		/* Definitions */
		if (arg instanceof PartDefinition)
			return createController((PartDefinition)arg);
		if (arg instanceof InterfaceDefinition)
			return createController((InterfaceDefinition)arg);
		if (arg instanceof AttributeDefinition)
			return createController((AttributeDefinition)arg);
		
		/* Types */
		if (arg instanceof FeatureType)
			return createController((FeatureType)arg);
		if (arg instanceof ComponentType)
			return createController((ComponentType)arg);
		if (arg instanceof AttributeValueType)
			return createController((AttributeValueType)arg);
		if (arg instanceof InterfaceType)
			return createController((InterfaceType) arg);
		
		/* Misc */
		if (arg instanceof BooleanExpression)
			return createController((BooleanExpression) arg);
		
		if (arg instanceof Connection)
			return createController((Connection) arg);
		
		
		return null;
	}

	public BasicController createConstraintGroupController(ComposableType type) {
		return new ConstraintGroupController(modelController, type);
	}

	public BasicController createConnectionGroupController(ComponentType type) {
		// TODO Auto-generated method stub
		return new  ConnectionGroupController(modelController, type);
	}
	
}
