/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import kumbang.core.model.exception.OnesOwnSupertypeException;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.ui.properties.FeatureTypePropertySource;

import org.eclipse.ui.views.properties.IPropertySource;

public class FeatureTypeController extends ComposableTypeController {


	public FeatureTypeController(ModelController modelController, FeatureType type) {
		super(modelController, type);
	
	}
	public Object[] getChildren() {
		Object[] parts = type.getPartsArray();
		
		
		
		
		//quick fix, should be redone like in ComponentTypeController
		Iterator<AttributeDefinition> attrIterator = type.getAttributes();   
		
		Vector<AttributeDefinition> attr = new Vector<AttributeDefinition>();
		
		while (attrIterator.hasNext())
			attr.add(attrIterator.next());
		
		Object[] attributes = attr.toArray();
		Object[] constraints = type.getConstraintsArray();
		Object[] children = new Object[parts.length + attributes.length + (type.hasConstraints() ? 1 : 0)];
		for(int i = 0; i < parts.length; i++) {
			children[i] = modelController.getControllerFactory().createController(parts[i]);
			if (children[i] instanceof PartDefinitionController)
				((PartDefinitionController)children[i]).setParent(this);
		}
		
		for(int i = 0; i < attributes.length; i++) {
			children[i + parts.length] = modelController.getControllerFactory().createController(attributes[i]);
			((AttributeDefinitionController)children[i + parts.length]).setParent(this);
		}
		
		if (type.hasConstraints())
			children[children.length - 1] = modelController.getControllerFactory().createConstraintGroupController(type);
		
//		for(int i = 0; i < constraints.length; i++) {
//			children[i + parts.length + attributes.length] = modelController.getControllerFactory().createController(constraints[i]);
//			if (children[i + parts.length + attributes.length] instanceof ConstraintController)
//				((ConstraintController)children[i + parts.length + attributes.length]).setParent(this);
//		}
		return children;
	}

	public boolean hasChildren() {
		if (type == null)
			return false;
		
		return type.getParts().hasNext() || type.hasConstraints() || type.getAttributes().hasNext();
	}

	public int getType() {
		return FEATURE_TYPE;
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new FeatureTypePropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}
	
	public String getImageString() {
			if (isAbstract())
				return BasicController.ABSTRACT_FEATURE_TYPE_IMAGE_STRING;
		
		return BasicController.FEATURE_TYPE_IMAGE_STRING;
	}
	
	public String getBackgroundString() {
		return FEATURE_TYPE_COLOR_STRING;
	}

	public String[] getPossibleFeatureReferences() {
		Iterator it = ((FeatureType)type).getParts();
		LinkedList list = new LinkedList();
		while(it.hasNext()) {
			PartDefinition def = (PartDefinition) it.next();
			if (def.getLowerBound() == 0)
				list.add(def);
		}
		String[] string = new String[list.size()];
		for (int i = 0; i < list.size(); i++)
			string[i] = ((PartDefinition)list.get(i)).getName();
		return string;
	}
	
	public String[] getPossibleAttributeReferences() {
		Iterator it = ((FeatureType)type).getAttributes();
		LinkedList list = new LinkedList();
		while(it.hasNext()) {
			AttributeDefinition def = (AttributeDefinition ) it.next();
			list.add(def);
		}
		String[] string = new String[list.size()];
		for (int i = 0; i < list.size(); i++)
			string[i] = ((AttributeDefinition)list.get(i)).getName();
		return string;
	}
	

	

	
	public void delete() {
		modelController.removeFeatureType(type.getName(), false);
		modified();
		
	}
	


}
