/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.modeller.ui.properties.InterfaceDefinitionPropertySource;

import org.eclipse.ui.views.properties.IPropertySource;

public class InterfaceDefinitionController extends BasicController {

	private InterfaceDefinition definition;
	private ComponentTypeController parent;
	
	public InterfaceDefinitionController(ModelController modelController, InterfaceDefinition definition) {
		super(modelController);
		this.definition = definition;

	}
	

	public boolean hasChildren() {
		return false;
		//return definition.getPossibleTypes().hasNext();
	}

	public String getDisplayText() {
		StringBuffer sb = new StringBuffer();
		List<InterfaceType> types = definition.getPossibleTypesList();
		if (types.size() == 1) {
			sb.append(types.get(0).getName());
		} else if (types.size() > 1){
			sb.append('(');
			for (int i = 0; i < types.size(); i++) {
				if (i != 0)
					sb.append(',');
				sb.append(types.get(i).getName());
			}
			sb.append(')');
		}
		sb.append(' ');
		
		if (definition.isGrounded() && definition.isOptional())
			return sb.toString() + definition.getName() + " {grounded, optional}";
		
		if (definition.isGrounded())
			return sb.toString() + definition.getName() + " {grounded}";
	
		if (definition.isOptional())
			return sb.toString() + definition.getName() + " {optional}" ;
		
		return sb.toString() + definition.getName();
	}

	public String getName() {
		return definition.getName();
	}

	public int getType() {
		return INTERFACE_DEFINITION;
	}

	public Object[] getChildren() {
		Object[] types = definition.getPossibleTypesArray();
		Object[] children = new Object[types.length];
		
		for (int i = 0; i < types.length; i++) {
			children[i] = modelController.getControllerFactory().createController(types[i]);
			((InterfaceTypeController)children[i]).setImageString(getImageString());
		}
		return children;
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new InterfaceDefinitionPropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public boolean isGrounded() {
		return definition.isGrounded();
	}

	public boolean isOptional() {
		return definition.isOptional();
	}

	public void setGrounded(boolean b) {
		if (b == isGrounded())
			return;
		definition.setGrounded(b);
		modified();
		
	}

	public void setOptional(boolean b) {
		if (b == isOptional())
			return;
		definition.setOptional(b);
		modified();
	}

	public String getImageString() {
		if (definition.isProvides())
			return INTERFACE_PROVIDES_IMAGE_STRING;
		return BasicController.INTERFACE_REQUIRES_IMAGE_STRING;
	}

	public void delete() {
		parent.removeInterfaceDefinition(definition.getName(), false);
		modified();
		
	}

	public void addPossibleType(InterfaceType interfaceType) {
		definition.addPossibleType(interfaceType);
		modified();
	}

	public void removeType(InterfaceType interfaceType) {
		definition.removePossibleType(interfaceType);
		modified();
	}

	public String getBackgroundString() {
		return INTERFACE_DEFINITION_COLOR_STRING;
	}

	public String[] getInterfaces() {
		Iterator it = definition.getPossibleTypes();
		LinkedList list = new LinkedList();
		while(it.hasNext()) {
			list.add(it.next());
		}
		String[] string = new String[list.size()];
		for (int i = 0; i < list.size(); i++)
			string[i] = ((InterfaceType)list.get(i)).getName();
		return string;
	}

	public boolean isProvides() {
		return definition.isProvides();
	}

	public void setName(String text) {
		if (definition.getName().equals(text))
			return;
		definition.setName(text);
		modified();
		
	}

	public void setProvides(boolean selection) {
		if (selection == isProvides())
			return;
		definition.setProvides(selection);
		modified();
		
	}

	public void setInterfaces(String[] selection) {
		List types = new ArrayList(selection.length);
		
		for (int i = 0; i < selection.length; i++) {
			InterfaceType interfaceType = modelController.getInterfaceType(selection[i]);
			if (!definition.getPossibleTypesList().contains(interfaceType))
				definition.addPossibleType(interfaceType);
		}
		
		modified();
	}

	public String getDisplayName() {
		return definition.getDisplayName();
		
	}

	public void setDisplayName(String text) {
		definition.setDisplayName(text);
		modified();

		
	}

	public int getParentType() {
		return COMPONENT_TYPE;
	}

	public String getParentName() {
		return parent.getName();
	}

	public void setParent(ComponentTypeController controller) {
		this.parent = controller;
		
	}
	
}
