/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.controllers;

import java.util.Iterator;
import java.util.LinkedList;

import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceType;
import kumbang.modeller.ui.properties.InterfaceTypePropertySource;

import org.eclipse.ui.views.properties.IPropertySource;

public class InterfaceTypeController extends BasicController {

	private InterfaceType type;
	private String imageString;

	public InterfaceTypeController(ModelController modelController,InterfaceType type) {
		super(modelController);
		this.type = type;
	}

	public Object[] getChildren() {
		return new Object[0];
	}

	public boolean hasChildren() {
		return false;
	}

	public String getName() {
		return type.getName();
	}

	public int getType() {
		return INTERFACE_TYPE;
	}

	public String getDisplayText() {
		return getName();
	}

	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (propertySource == null)
				propertySource = new InterfaceTypePropertySource(this, modelController);
			return propertySource;
		}
		return null;
	}

	public String getImageString() {
		if (imageString != null)
			return imageString;
		return BasicController.INTERFACE_TYPE_IMAGE_STRING;
	}

	public void delete() {
		modelController.removeInterfaceType(type.getName(), false);
	}

	public String getBackgroundString() {
		return INTERFACE_TYPE_COLOR_STRING;
	}

	public String[] getFunctions() {
		Iterator<String> it = type.getFunctions();
		LinkedList<String> list = new LinkedList<String>();
		while(it.hasNext()) {
			list.add(it.next());
		}
		return list.toArray(new String[list.size()]);
	}

	public void setName(String newName) {
		if (getName().equals(newName))
			return;
		modelController.updateTypeName(type, newName);
		
	}

	public void setFunctions(String[] items) throws Exception {
		type.setFunctions(items);
		modified();
		
	}

	public void setImageString(String imageString) {
		this.imageString = imageString;
		
	}
	
	public String getDisplayName() {
			return type.getDisplayName();
	
	}

	public void setDisplayName(String text) {
		type.setDisplayName(text);
		modified();
	}
}
