/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/**
 * @author vmyllarn
 */
package kumbang.modeller.modelchecker;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;

import java.util.ArrayList;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.jface.dialogs.MessageDialog;

import com.sun.org.apache.bcel.internal.generic.GETSTATIC;

//import kumbang.configurator.server.core.ServerProperties;
import kumbang.configurator.server.smodels.ConfigurationState;
import kumbang.configurator.server.smodels.EngineInterface;
import kumbang.configurator.server.smodels.Translator;

import kumbang.core.language.parser.ModelParser;
import kumbang.core.language.parser.ParserInterface;

import kumbang.core.model.core.Model;

import kumbang.core.model.exception.ModelException;
import kumbang.modeller.KumbangModellerPlugin;

/*
 * TODO: The .lp and .sm files are created to the eclipse root directory and not deleted 
 * afterwards
 */

public class ModelChecker {

	private static final String BIN_PATH = "bin";

	private EngineInterface engine;
	
	private ParserInterface modelParser;
	
	public ModelChecker() throws IOException {
		// Set the proper root directory value, because EngineInterface will
		// ask for it.
		// TODO: Maybe to this somewhere else during startup?
		//ServerProperties.getInstance().setRootDir(getPluginFolder().toString());

		this.engine = new EngineInterface();
		this.modelParser = new ModelParser();
	}
	
	/**
	 * Checks given model in a file for consistency.
	 * @param file Kumbang model file
	 * @return false if no configuration cannot be found, true otherwise
	 * @throws ModelException thrown if model was not semantically valid
	 * @throws IOException thrown if model file was not OK
	 */
	public boolean checkModel(Model model) throws ModelException, IOException {
		String lparseContent = Translator.Translate(model);
		File lpFile = new File(model.getName() + ".lp");
		PrintWriter out = new PrintWriter(new FileWriter(lpFile));
		out.println(lparseContent);
		out.close();

		File smFile = new File(model.getName() + ".sm");
		ground(lpFile, smFile);

		this.engine.init(smFile);
		engine.resetComputeStatement(new ArrayList(), new ArrayList());
		ConfigurationState state = engine.getConfigurationState();

		// TODO: Is it ok to delete these?
		smFile.delete();
		lpFile.delete();

		return state.isConsistent();
	}
	
	public boolean checkModel(File file) throws ModelException, IOException {
		Model model = modelParser.parseModel(file);
		return checkModel(model);
	}

	/*
	 * Get the path to this plugin 
	 */
	private static File getPluginFolder() throws IOException {
		URL url = KumbangModellerPlugin.getDefault().getBundle().getEntry("/");
		url = FileLocator.toFileURL(url);
		System.out.println(url.getPath());
		return new File(url.getPath());
	}
	
	private static void ground(File lparseFile, File smodelsFile) throws IOException {
		// Path to the lparse binaries
		// File binPath = new File(getPluginFolder(), BIN_PATH);
		File binPath = new File(kumbang.core.Activator.getPluginDir("kumbang.core"),"bin");
		String executable = "";
		if (System.getProperties().getProperty("os.name").matches(
				"(?i).*windows.*")) {
			executable = "lparse.exe";
		} else {
			executable = "lparse";
		}
		
		PrintWriter out = new PrintWriter(new FileWriter(smodelsFile));
	
		String cmd1 = new File(binPath, executable).toString();
		String cmd2 = lparseFile.toString();
		
		String[] arrayCmd = new String[] { cmd1, cmd2 };
		Process process = Runtime.getRuntime().exec(arrayCmd);
		
		BufferedReader in = new BufferedReader(new InputStreamReader(process
				.getInputStream()));
		String line;
		while ((line = in.readLine()) != null) {
			out.println(line);
		}
		out.close();
	}
}
