/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.AttributeDefinitionController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.operations.AddAttributeDefinitionOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class AttributeDefinitionDialog extends DefinitionDialog  {


	private Combo types;
	

	private String defaultAttribute;

	private AttributeDefinitionController controller;

	public AttributeDefinitionDialog(Shell shell, IUndoContext undoContext, ComposableTypeController type, ModelController modelController) {
		super(shell, undoContext, type, modelController, "Add Attribute");		
	}

	public AttributeDefinitionDialog(Shell shell, IUndoContext undoContext, ComposableTypeController type, ModelController modelController, AttributeDefinitionController controller) {
		super(shell, undoContext, type, modelController, "Attribute Definition");		
		this.controller = controller;
		nameReady = true;
		updateStatus(Status.OK_STATUS);
	}
	
	
	public AttributeDefinitionDialog(IUndoContext undoContext, ModelController modelController, ComposableTypeController controller, String name) {
		super(null, undoContext, controller, modelController, "Add Attribute");
		this.defaultAttribute = name;
		
	}


	protected Control createDialogArea(Composite parent) {
		Composite composite =(Composite)super.createDialogArea(parent);
		createNameText(composite, "Attribute Definition Name:", controller);
		createAttributeTypesArea(composite);
	
		return composite;
	}
	
	

	
	private void createAttributeTypesArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);	
		composite.setLayout(new GridLayout(1, true));
		new Label(composite, SWT.NULL).setText("Attribute Type:");
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		types =  new Combo(composite, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY);
		types.setItems(modelController.getAttributeTypes());

		types.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		if (controller != null) {
			String att = controller.getAttributeName();
			String[] items =types.getItems();
			for (int i = 0; i < items.length; i++) {
				if (items[i].equals(att)) {
					types.select(i);
					break;
				}
			}
		}
		else if (defaultAttribute != null){
			setAttribute(defaultAttribute);
		
		
			
		}
	
		Button create = new Button(composite, SWT.NULL);
		
		create.setText("Create New Type...");
		create.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
				new AttributeValueTypeDialog(getShell(), undoContext, modelController, AttributeDefinitionDialog.this).open();
				
			}});
		
//		types.addSelectionListener(new SelectionAdapter(){
//
//			public void widgetDefaultSelected(SelectionEvent e) {
//				types.getItem(types.getSelectionIndex());
//				
//			}});
	}


	protected void updateInfo() {
		controller.setName(nameText.getText());
		controller.setDisplayName(displayNameText.getText());
		controller.setTypes(types.getItem(types.getSelectionIndex()));
		
	}

	protected  void doNew() throws ExecutionException {
		
		doOperation(new AddAttributeDefinitionOperation(typeController, new Name(nameText.getText(), displayNameText.getText()), types.getItem(types.getSelectionIndex())));
	
		
	}

	

	public void widgetDefaultSelected(SelectionEvent e) {
	
	}


	protected void checkStatus() {
		if (nameReady)
			updateStatus(Status.OK_STATUS);
		else updateStatus(status);
		
	}


	protected boolean isPropertyDialog() {
		return controller != null;
	}

	public void addAttribute(String name) {
		this.types.setItems(modelController.getAttributeTypes());
		setAttribute(name);
		
	}
	
	public void setAttribute(String attribute) {
		
		String[] items = types.getItems();
		for (int i = 0; i < items.length; i++) {
			if (items[i].equals(attribute)) {
				types.select(i);
				break;
			}
		}
		
		if (!nameReady) {
			nameText.setText(attribute.toLowerCase());
			nameReady = true;
		}
	}
}
