/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import java.util.HashMap;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.AttributeValueTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.operations.NewAttributeValueTypeOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class AttributeValueTypeDialog extends TypeDialog {

	private class ValueDialog extends NameTextUndoableOperationDialog {

	
		
		private Name value;
		private int index = values.getSelectionIndex();
		
		protected ValueDialog(Shell parentShell) {
			super(parentShell, null, "Add a Value");
			
		}
		
		protected ValueDialog(Shell parentShell, Name value) {
			super(parentShell, null, "Edit a Value");
			this.value = value;
			nameReady = true;
			
		}
		
		protected Control createDialogArea(Composite parent) {
			Composite composite = new Composite(parent, SWT.NULL);	
			composite.setLayout(new GridLayout(1, true));
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			createNameText(composite, "Value", null);
			if (value != null) {
				nameText.setText(value.getName());
				if (value.hasDisplayName())
					displayNameText.setText(value.getDisplayName());
			}
			return composite;
		}
		
	
		
		@Override
		protected void checkStatus() {
			if (nameReady)
				updateStatus(Status.OK_STATUS);
			
		}
		@Override
		protected void doNew() throws ExecutionException {
			Name name = new Name(nameText.getText(), displayNameText.getText());
			names.put(name.getDisplayName(), name);
			values.add(name.getDisplayName());
			AttributeValueTypeDialog.this.checkStatus();
		}
		@Override
		protected boolean isPropertyDialog() {
			return  value != null;
		}
		@Override
		protected void updateInfo() {
			names.remove(value.getDisplayName());
			values.remove(value.getDisplayName());
			Name name = new Name(nameText.getText(), displayNameText.getText());
			names.put(name.getDisplayName(), name);
			values.add(name.getDisplayName(), index);
			AttributeValueTypeDialog.this.checkStatus();
		}
		
	}
	



	private List values;
	private HashMap<String,Name> names = new HashMap<String, Name>();
	private AttributeValueTypeController controller;
	private Button add;
	private Button edit;
	private Button remove;
	private AttributeDefinitionDialog dialog;


	public AttributeValueTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController) {
		super(shell, undoContext, modelController, "New Attribute Value Type");

	}
	
	public AttributeValueTypeDialog(Shell shell, ModelController modelController, AttributeValueTypeController controller) {
		super(shell, null, modelController, "Attribute Value Type");
		this.controller = controller;
		nameReady = true;
		updateStatus(Status.OK_STATUS);

	}
	
	public AttributeValueTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, AttributeDefinitionDialog dialog) {
		this(shell, undoContext, modelController);
		this.dialog = dialog;
	}

	protected Control createDialogArea(Composite parent) {

		
		Composite composite = (Composite) super.createDialogArea(parent);
		
		
		createNameText(composite, "Atrribute Name:", controller);
		
		Composite valuesComposite = new Composite(composite, SWT.NULL);
		valuesComposite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		valuesComposite.setLayout(LayoutFactory.getInstance().getFormLayout());
		Label label = new Label(valuesComposite, SWT.NULL);
		label.setText("Values:");
		//label.setLayoutData(new GridData (SWT.LEFT, SWT.CENTER, false, false, 3, 1));
	
	   
		values = new List(valuesComposite, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);  
	
		//values.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 3));
	    
	    
	    values.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				new ValueDialog(AttributeValueTypeDialog.this.getShell(), names.get(values.getItem(values.getSelectionIndex()))).open();
				
			}

			public void widgetSelected(SelectionEvent e) {
				updateButtonStatus();
				
			}});
	    
	    if (controller != null) {
	    	Name[] names = controller.getValues();
	    	for (int i = 0; i <  names.length; i++) {
	    		this.names.put(names[i].getDisplayName(), names[i]);
	    		values.add(names[i].getDisplayName());
	    	}  	
	    } 
	    
	    add = new Button(valuesComposite, SWT.NULL);
	    add.setText("Add...");
	    add.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent e) {
	    		new ValueDialog(AttributeValueTypeDialog.this.getShell()).open();
	    		
	    	}
	    	
	    });
	    
	   
	    
	    edit = new Button(valuesComposite, SWT.NULL);
	    edit.setText("Edit...");  
	    //edit.setLayoutData(buttonData);
	
	    
	    edit.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent e) {
	    		if (values.getSelectionCount() == 0)
	    			return;
	    		Name name = names.get(values.getSelection()[0]);
	    		new ValueDialog(AttributeValueTypeDialog.this.getShell(), name).open();
	    	
	    	}
	    	
	    });
	    
	    
	    remove = new Button(valuesComposite, SWT.NULL);
	    remove.setText("Remove");
	 ;
	    
	    remove.addSelectionListener(new SelectionAdapter() { 
	    	public void widgetSelected(SelectionEvent e) {
	    		String[] selected = values.getSelection();
	    		for (int i = 0; i <  selected.length; i++) {
	    			names.remove(selected[i]);
	    			values.remove(selected[i]);
	    		 }
	    		updateButtonStatus();
	    		
	    	}
	    	
	    });
	    

	    
	    values.setLayoutData(LayoutFactory.getInstance().getCenterData(label));
	    add.setLayoutData(LayoutFactory.getInstance().getLeftButtonData(3, values));
	    edit.setLayoutData(LayoutFactory.getInstance().getMiddleButtonData(add, remove, values));
	    remove.setLayoutData(LayoutFactory.getInstance().getRightButtonData(3, values));
	   
	    updateButtonStatus();
	    return composite;
	
		
	}

	
	private void updateButtonStatus() {
		if (values.getSelectionCount() != 0) {
			if (values.getSelectionCount() == 1)
				edit.setEnabled(true);
			else 
				edit.setEnabled(false);
			remove.setEnabled(true);
			
		} else {
			edit.setEnabled(false);
			remove.setEnabled(false);
		}
	}
	
	protected void checkStatus() {
		if (nameReady && values.getItemCount() > 0)
			updateStatus(Status.OK_STATUS);
		else updateStatus(status);
		
	}

	protected void updateInfo() {
		controller.setName(nameText.getText());
		controller.setDisplayName(displayNameText.getText());
		try {
			controller.setValues(names);
		} catch (PropertyAlreadyDefinedException e) {
			status.setMessage(e.getMessage());
			status.setThrowable(e);
			updateStatus(status);
		}
		
	}

	protected void doNew() throws ExecutionException {
		doOperation(new NewAttributeValueTypeOperation(modelController, new Name(nameText.getText(), displayNameText.getText()), names));
		if (dialog != null)
			dialog.addAttribute(nameText.getText());
		
	}

	protected boolean isPropertyDialog() {
		return controller != null;
	}

	

}
