/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import java.util.HashMap;
import java.util.Iterator;

import kumbang.core.model.exception.OnesOwnSupertypeException;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.operations.NewComposableTypeOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class ComposableTypeDialog extends TypeDialog {


	private class SupertypesDialog extends Dialog {

		private List possibleSupertypes;

		protected SupertypesDialog(Shell parentShell) {
			super(parentShell);
			
		}
		
		protected Control createDialogArea(Composite parent) {
			Composite composite = new Composite(parent, SWT.NULL);
			composite.setLayout(new GridLayout(1, true));
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			possibleSupertypes = new List(composite, SWT.BORDER | SWT.MULTI);
			if (isFeature)
				possibleSupertypes.setItems(modelController.getFeatureTypes());
			else possibleSupertypes.setItems(modelController.getComponentTypes());
			
			if (controller != null && possibleSupertypes.getItemCount() > 0 ) {
				possibleSupertypes.remove(controller.getDisplayName());
			}
			
			for (int i = 0; i < supertypes.getItemCount(); i++) {
				possibleSupertypes.remove(supertypes.getItem(i));
			}
			possibleSupertypes.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			Button create = new Button(composite, SWT.NULL);
			create.setText("Create New " + (isFeature ? "Feature Type..." : "Component Type"));
			create.setLayoutData(LayoutFactory.getInstance().getButtonGridData());
		
			create.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
				
						new ComposableTypeDialog(getShell(), undoContext, modelController, ComposableTypeDialog.this, isFeature).open();
					
					SupertypesDialog.this.close();
				}});
			return composite;
			
			
		}

		protected void okPressed() {
			String[] selection = possibleSupertypes.getSelection();
			for (int i = 0; i < selection.length; i++) {
				supertypes.add(selection[i]);
				ComposableType type;
				if (isFeature)
					type = modelController.getFeatureType(selection[i]);
				else type = modelController.getComponentType(selection[i]);
				
				typeMap.put(selection[i], type);
			}
			
			super.okPressed();
		}


	}

	private ComposableTypeController controller;
	private List supertypes;
	private Button isAbstract;
	protected boolean abstractModified;
	private Button add;
	private Button remove;
	private HashMap<String, ComposableType> typeMap = new HashMap<String, ComposableType>();
	private NameTextUndoableOperationDialog dialog;
	private boolean isFeature;
	private boolean isRoot;
	
	
	public ComposableTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, boolean isFeature) {
		super(shell, undoContext, modelController, isFeature ? "New Feature Type" : "New Component Type");
		this.isFeature = isFeature;
		
	}
	
	public ComposableTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, ComposableTypeController controller, boolean isFeature) {
		super(shell, undoContext, modelController, isFeature ? "Feature Type" : "Component Type");
		this.controller = controller;
		this.isFeature = isFeature;
		nameReady = true;
		updateStatus(Status.OK_STATUS);
	}




	
	public ComposableTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, NameTextUndoableOperationDialog dialog, boolean isFeature) {
		this(shell, undoContext, modelController, isFeature);
		this.dialog = dialog;
	}

	public ComposableTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController,  boolean isFeature, boolean isRoot) {
		
		this(shell, undoContext, modelController, isFeature);
		this.isRoot = isRoot;
	}

	protected Control createDialogArea(Composite parent) {
	
		Composite composite = (Composite) super.createDialogArea(parent);
		composite.setLayout(new GridLayout());
		createNameText(composite, isFeature ? "Feature Type Name:" : "Component Type Name", controller);
	 
	    Composite superTypesComposite  = new Composite(composite, SWT.NULL);
		superTypesComposite.setLayout(LayoutFactory.getInstance().getFormLayout());
		superTypesComposite.setLayoutData(LayoutFactory.getInstance().getFillGridData());

	    
		Label label =  new Label(superTypesComposite, SWT.NULL);
		label.setText("Supertypes:");
		
		 supertypes = new List(superTypesComposite, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL);
	  
		 supertypes.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				

			}

			public void widgetSelected(SelectionEvent e) {
				updateButtonStatus();
				
			}});
	    
	    
	    add = new Button(superTypesComposite, SWT.NULL);
	    add.setText("Add..");
	    add.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent e) {
	    		new SupertypesDialog(getShell()).open();
	    	}
	    	
	    });
	    
	    if (controller != null) {
	    	Iterator<ComposableType> it = controller.getSupertypes();
	    	while (it.hasNext()) {
	    		ComposableType next = it.next();
	    		supertypes.add(next.getDisplayName());
	    		typeMap.put(next.getDisplayName(), next);
	    		
	    	
	    	}
	    	//add.setEnabled(modelController.getFeatureTypeCount() > 1);
	    } //else    add.setEnabled(modelController.getFeatureTypeCount() > 0);

	 

	    
	    remove = new Button(superTypesComposite, SWT.NULL);
	    remove.setText("Remove");
	  
	    
	    remove.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent e) {
	    		String[] selection = supertypes.getSelection();
	    		for (int i = 0; i < selection.length; i++) {
	    			supertypes.remove(selection[i]);
	    			typeMap.remove(selection[i]);
	    		}
	    		
	    		updateButtonStatus();
	    	}
	    	
	    });
	
	  
	    
	    Composite abstractComposite = new Composite(composite, SWT.NULL);
	    // A quick hack to get the butten in line with rest of the elements
	    GridLayout gl = new GridLayout();
	    gl.marginWidth = 8;
	 
	    abstractComposite.setLayout(gl);
	    abstractComposite.setLayoutData(new GridData(GridData.END));
	    isAbstract = new Button(abstractComposite, SWT.CHECK);
	    isAbstract.setText("Type is abstract");
	   // isAbstract.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
	   // isAbstract.setLayoutData(new GridData(GridData.END));
	   // isAbstract.addSelectionListener(new SelectionAdapter() {

		//	public void widgetSelected(SelectionEvent e) {
		//		abstractModified = true;
		//		checkStatus();
				
			//}
		//});
	    //if (controller != null)
	    //	isAbstract.setSelection(controller.isAbstract());
	  
	    supertypes.setLayoutData(LayoutFactory.getInstance().getCenterData(label));
	    
	    add.setLayoutData(LayoutFactory.getInstance().getLeftButtonData(2, supertypes));
	    remove.setLayoutData(LayoutFactory.getInstance().getRightButtonData(2, supertypes));
  
	    
	    updateButtonStatus();
	    return composite;
	
		
	}

	private void updateButtonStatus() {
		if (supertypes.getSelectionCount() == 0) 
			remove.setEnabled(false);
		else remove.setEnabled(true);
	}
	
    protected void checkStatus() {
		updateStatus(Status.OK_STATUS);
		
	}

	protected void updateInfo() throws OnesOwnSupertypeException  {
		controller.setName(nameText.getText());
		controller.setSupertypes(typeMap.values());
		controller.setDisplayName(displayNameText.getText());
		if (abstractModified)
			controller.setAbstract(isAbstract.getSelection());
		
	}

	protected void doNew() throws ExecutionException {
		doOperation(new NewComposableTypeOperation(modelController, new Name(nameText.getText(), displayNameText.getText()), typeMap.values(), isAbstract.getSelection(), isFeature, isRoot));
		if (dialog != null) {
			if (dialog instanceof PartDefinitionDialog) {
			 ((PartDefinitionDialog)dialog).addPartName(nameText.getText());
			} else if (dialog instanceof ComposableTypeDialog)
				 ((ComposableTypeDialog)dialog).supertypes.add(nameText.getText());
		}
	}

	protected boolean isPropertyDialog() {
		return controller != null;
	}

}
