/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import java.util.Arrays;

import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.operations.AddInterfaceDefinitionOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class InterfaceDefinitionDialog extends DefinitionDialog implements SelectionListener {

	
	private class InterfacesDialog extends Dialog {

		private List interfacesList;

		protected InterfacesDialog(Shell parentShell) {
			super(parentShell);			
		}
		
		protected Control createDialogArea(Composite parent) {
			Composite composite = new Composite(parent, SWT.MULTI);
			composite.setLayout(new GridLayout(1, true));
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			interfacesList = new List(composite, SWT.BORDER | SWT.V_SCROLL );
			interfacesList.setItems(modelController.getInterfaces());
			interfacesList.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			for (int i = 0; i < interfaces.getItemCount(); i++)
				interfacesList.remove(interfaces.getItem(i));
			
			Button create = new Button(composite, SWT.NULL);
			create.setText("Create New Interface Type...");
			create.setLayoutData(LayoutFactory.getInstance().getButtonGridData());
		
			create.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					new InterfaceTypeDialog(getShell(), undoContext, modelController, InterfaceDefinitionDialog.this).open();
					InterfacesDialog.this.close();
				}	
						
			});
			
			return composite;
		}

		protected void okPressed() {
			String[] selection = interfacesList.getSelection();
			for (int i = 0; i < selection.length; i++) {
				interfaces.add(selection[i]);
			}
			super.okPressed();
		}


	}
	

	private List interfaces;
	private Button grounded;
	private Button optional;
	private Button provides;
	private Button requires;
	
	private boolean typesReady = false;
	private boolean providesReady = false;
	private String defaultType;
	private InterfaceDefinitionController controller;
	private Button remove;
	

	public InterfaceDefinitionDialog(Shell shell, IUndoContext undoContext, ComposableTypeController typeController, ModelController modelController) {
		super(shell, undoContext, typeController, modelController, "Add Interface Definition");
		
	}
	
	public InterfaceDefinitionDialog(Shell shell, IUndoContext undoContext, ComposableTypeController typeController, ModelController modelController, InterfaceDefinitionController definitionController) {
		super(shell, undoContext, typeController, modelController, "Interface Definition");
		this.controller = definitionController;
		nameReady = true;
		typesReady = true;
		providesReady = true;
		updateStatus(Status.OK_STATUS);
	}
	
	public InterfaceDefinitionDialog(IUndoContext undoContext, ModelController modelController, ComposableTypeController controller, String name) {
		this(null, undoContext, controller, modelController);
		this.defaultType = name;
		typesReady = true;
	}

	protected Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);	
		composite.setLayout(new GridLayout(1, true));
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
	
		createNameText(composite, "Definition Name:", controller);	
		createInterfaceSelection(composite);
		createInterfaceDirection(composite);		
		createAdditionalChoices(composite);
		
		return composite;
	}
	
	private void createInterfaceDirection(Composite composite) {
		Group group = new Group(composite, SWT.NULL);
		group.setLayout(new GridLayout(1, true));
		group.setText("Interface Direction");
		group.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		provides = new Button(group, SWT.RADIO);
		provides.setText("Provides");
		
		requires = new Button(group, SWT.RADIO);
		requires.setText("Requires");
		
		if (controller != null) {
			boolean isProvides = controller.isProvides();
			provides.setSelection(isProvides);
			requires.setSelection(!isProvides);
		}
		
		provides.addSelectionListener(this);
		requires.addSelectionListener(this);
		
	}

	private void createAdditionalChoices(Composite composite) {
		Group group = new Group(composite, SWT.NULL);
		group.setLayout(new GridLayout(1, true));
		group.setText("Additional Options");
		group.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
	
		grounded = new Button(group, SWT.CHECK);
		grounded.setText("Grounded");
		grounded.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		
		optional = new Button(group, SWT.CHECK);
		optional.setText("Optional");
		optional.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		if (controller != null) {
			grounded.setSelection(controller.isGrounded());
			optional.setSelection(controller.isOptional());
		}
		
	}

	private void createInterfaceSelection(Composite composite) {
		
		Composite interfaceComposite = new Composite(composite, SWT.NULL);
		
		Label label = new Label(interfaceComposite, SWT.NULL);
		label.setText("Interface Types:");
		
		
		interfaceComposite.setLayout(LayoutFactory.getInstance().getFormLayout());
		interfaceComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
		
		
		interfaces =  new List(interfaceComposite, SWT.MULTI | SWT.BORDER |  SWT.V_SCROLL );
		if (defaultType != null) {
			interfaces.add(defaultType);
			
			
			nameText.setText(defaultType.toLowerCase());
			nameReady = true;
			
		}
		else if (controller != null) {
			interfaces.setItems(controller.getInterfaces());
		}
	
		
		interfaces.addSelectionListener(this);
		
		Button add = new Button(interfaceComposite, SWT.NULL);
		add.setText("Add..");
		//add.setLayoutData(LayoutFactory.getInstance().getButtonGridData());
	
		
		add.addSelectionListener(new SelectionAdapter() {
		    	public void widgetSelected(SelectionEvent e) {
		    		new InterfacesDialog(InterfaceDefinitionDialog.this.getShell()).open();
		    		updateButtonStatus();
		    	}
		    	
		    });
		
		
		remove = new Button(interfaceComposite, SWT.NULL);
		remove.setText("Remove");
		remove.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent e) {
	    		interfaces.remove(interfaces.getSelectionIndices());
	    		updateButtonStatus();
	    	}
	    	
	    });


		interfaces.setLayoutData(LayoutFactory.getInstance().getCenterData(label));
	    add.setLayoutData(LayoutFactory.getInstance().getLeftButtonData(2, interfaces));
	    remove.setLayoutData(LayoutFactory.getInstance().getRightButtonData(2, interfaces));
	
	    
	    updateButtonStatus();
	}
	
	
	private void updateButtonStatus() {
		if (interfaces.getItemCount() > 0)
			typesReady = true;
		else typesReady = false;
		
		
		if (interfaces.getSelectionCount() == 0) 
			remove.setEnabled(false);
		else remove.setEnabled(true);
		
		checkStatus();
	}
	
	public void widgetSelected(SelectionEvent e) {
		if (e.getSource() == provides || e.getSource() == requires) 
			providesReady = true;
		else if (e.getSource() == interfaces)
			typesReady = true;
		updateButtonStatus();
		checkStatus();
	}

	public void widgetDefaultSelected(SelectionEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void checkStatus() {
		if (nameReady && typesReady && providesReady)
			updateStatus(Status.OK_STATUS);
	}

	protected void updateInfo() {
		controller.setName(nameText.getText());
		controller.setDisplayName(displayNameText.getText());
		controller.setGrounded(grounded.getSelection());
		controller.setOptional(optional.getSelection());
		controller.setProvides(provides.getSelection());
		controller.setInterfaces(interfaces.getItems());
		
	}

	protected void doNew() throws ExecutionException {
		doOperation(new AddInterfaceDefinitionOperation( typeController, new Name(nameText.getText(), displayNameText.getText()), interfaces.getItems(), grounded.getSelection(), optional.getSelection(), provides.getSelection()));
		
	}

	protected boolean isPropertyDialog() {
		return controller != null;
	}

	public void addInterfaceName(String name) {
		interfaces.add(name);
		
		if (!nameReady) {
			nameText.setText(name.toLowerCase());
			nameReady = true;
		}
		
	}
}
