/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.InterfaceTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.operations.NewInterfaceTypeOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class InterfaceTypeDialog extends TypeDialog {

	private class AddFunctionDialog extends Dialog {

		private Text function;
		private String text;
		
		
		protected AddFunctionDialog(Shell parentShell) {
			super(parentShell);
			
			
		}
		public AddFunctionDialog(Shell shell, String text) {
			super(shell);
			this.text = text;
		}
		protected Control createDialogArea(Composite parent) {
			
			
			Composite composite = new Composite(parent, SWT.NULL);	
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			composite.setLayout(new GridLayout(1, true));
			
			new Label(composite, SWT.NULL).setText("Function:");
			function = new Text(composite, SWT.BORDER | SWT.SINGLE);
			function.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
			if (text != null)
				function.setText(text);
			return composite;
		}
		
		protected void okPressed() {
			if (text != null) {
				int index = functions.getSelectionIndex();
				functions.remove(text);
				functions.add(function.getText(), index);
			} else {			
				functions.add(function.getText());
			}
			super.okPressed();
		}
		
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			if (text == null)
				newShell.setText("Add Function");
			else 
				newShell.setText("Edit Function");
			    
		}
		
	}
	
	private class AddInterfacesDialog extends Dialog {

		private ModelController modelController;
		private List interfaces;
		
		protected AddInterfacesDialog(Shell parentShell, ModelController controller) {
			super(parentShell);
			this.modelController = controller;
			
		}
		
		protected Control createDialogArea(Composite parent) {
			Composite composite = new Composite(parent, SWT.NULL);	
			composite.setLayout(new GridLayout(1, true));
		
			interfaces = new List(composite, SWT.BORDER | SWT.MULTI);
			interfaces.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			interfaces.setItems(modelController.getInterfaces());
			
			return composite;
		}
		
		protected void okPressed() {
			String[] selected = interfaces.getSelection();
			for (int i = 0; i < selected.length; i++)
				ancestors.add(selected[i]);
			updateButtonStatus();
			super.okPressed();
		}
		
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText("Add Interfaces");
			    
		}
		
	}
	
	
		private List ancestors;
		private List functions;

		private InterfaceTypeController controller;
		private Button add;
		private Button edit;
		private Button remove;
		private InterfaceDefinitionDialog dialog;
		
		public InterfaceTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController) {
			super(shell, undoContext, modelController, "New Interface Type");
			
		}
		
		public InterfaceTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, InterfaceTypeController controller) {
			super(shell, undoContext, modelController, "Interface Type");
			this.controller = controller;
			nameReady = true;
			updateStatus(Status.OK_STATUS);
		}


		public InterfaceTypeDialog(Shell shell, IUndoContext undoContext, ModelController modelController, InterfaceDefinitionDialog dialog) {
			this(shell, undoContext, modelController);
			this.dialog = dialog;
		}

		protected Control createDialogArea(Composite parent) {
	
			Composite composite = (Composite) super.createDialogArea(parent);
			composite.setLayout(new GridLayout(1, true));
			createNameText(composite, "Interface Name:", controller);
//		    new Label(composite, SWT.NULL).setText("Ancestors:");
//		    ancestors = new List(composite, SWT.SINGLE | SWT.BORDER);
//		    GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
//		    ancestors.setLayoutData(data);
//		    Button browse = new Button(composite, SWT.NULL);
//		    browse.setText("Add..");
//		    browse.addSelectionListener(new SelectionAdapter() {
//		    	
//		    	public void widgetSelected(SelectionEvent e) {
//		    		new AddInterfacesDialog(getShell(), modelController).open();
//		    	}
//		    	
//		    });
		    

		   createFunctions(composite);
		
			return composite;
		}
	
		private void createFunctions(Composite parent) {
			Composite composite = new Composite(parent, SWT.NULL);
			
			composite.setLayout(LayoutFactory.getInstance().getFormLayout());
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			Label label = new Label(composite, SWT.NULL);
			label.setText("Functions:");
			//label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
			
			functions = new List(composite, SWT.MULTI| SWT.BORDER | SWT.V_SCROLL);
			//functions.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 5));
			
			functions.addSelectionListener(new SelectionListener(){

				public void widgetDefaultSelected(SelectionEvent e) {
					// TODO Auto-generated method stub
					
				}

				public void widgetSelected(SelectionEvent e) {
					updateButtonStatus();
				}
					
				});
			
			if (controller != null)
				functions.setItems(controller.getFunctions());
		
			
			add = new Button(composite, SWT.NULL);
			add.setText("Add..");
			
			add.addSelectionListener(new SelectionAdapter() {
			    	
			    	public void widgetSelected(SelectionEvent e) {
			    		new AddFunctionDialog(InterfaceTypeDialog.this.getShell()).open();
			    	
			    	}
			    	
			    });
			
			edit = new Button(composite, SWT.NULL);
			edit.setText("Edit...");
			edit.addSelectionListener(new SelectionAdapter() {
		    	
		    	public void widgetSelected(SelectionEvent e) {
		    		new AddFunctionDialog(InterfaceTypeDialog.this.getShell(), functions.getItem(functions.getSelectionIndex())).open();
		    	
		    	}
		    	
		    });
			
			remove = new Button(composite, SWT.NULL);
			remove.setText("Remove");
	
			remove.addSelectionListener(new SelectionAdapter() {
		    	
		    	public void widgetSelected(SelectionEvent e) {
		    		functions.remove(functions.getSelectionIndices());
		    		updateButtonStatus();
		    		
		    	}
		    	
		    });
			
			functions.setLayoutData(LayoutFactory.getInstance().getCenterData(label));
		    add.setLayoutData(LayoutFactory.getInstance().getLeftButtonData(3, functions));
		    edit.setLayoutData(LayoutFactory.getInstance().getMiddleButtonData(add, remove, functions));
		    remove.setLayoutData(LayoutFactory.getInstance().getRightButtonData(3, functions));
	
		    updateButtonStatus();
		}

		private void updateButtonStatus() {
			if (functions.getSelectionCount() == 0) {
				remove.setEnabled(false);
				edit.setEnabled(false);
			} else if (functions.getSelectionCount() == 1) {
				remove.setEnabled(true);
				edit.setEnabled(true);
			} else {
				edit.setEnabled(false);
				remove.setEnabled(true);
			}
		}
		

		protected void checkStatus() {
			updateStatus(Status.OK_STATUS);
			
		}

		protected void updateInfo() throws Exception {
			controller.setName(nameText.getText());
			controller.setDisplayName(displayNameText.getText());
		
				controller.setFunctions(functions.getItems());
		
			
		}

		protected void doNew() throws ExecutionException {
			doOperation(new NewInterfaceTypeOperation(modelController, new Name(nameText.getText(), displayNameText.getText()), functions.getItems()));
			if (dialog != null) {
				dialog.addInterfaceName(nameText.getText());
			}
		}

		protected boolean isPropertyDialog() {
			return controller != null;
		}
	



	
}
