/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Layout;

public class LayoutFactory {

	private static FormLayout formLayout;
	private static RowLayout rowLayout;
	
	private static GridData fillData = new GridData(GridData.FILL_BOTH);
	private static GridData fillDataHorizontal = new GridData(GridData.FILL_HORIZONTAL);
	private static GridData buttonData = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
	
	private static int width = 5;
	private static LayoutFactory instance;
	
	private LayoutFactory() {}
	
	public static LayoutFactory getInstance() {
		if (instance == null)
			instance = new LayoutFactory();
		return instance;
	}
	
	public Layout getFormLayout() {
		if (formLayout == null) {
			formLayout = new FormLayout();
			formLayout.marginWidth = width;
		}
		return formLayout;
	}

	public GridData getFillGridData() {
		return fillData;
	}
	
	public GridData getHorizontalFillGridData() {
		return fillDataHorizontal;
	}
	
	public GridData getButtonGridData() {
		return buttonData;
	}

	public Layout getRowLayout() {
		if (rowLayout != null)
			return rowLayout;
		rowLayout = new RowLayout(SWT.VERTICAL);
		rowLayout.fill = true;
		rowLayout.marginWidth = width;
		//rowLayout.justify = true;
		return rowLayout;
	}

	public FormData getLeftButtonData(int buttons, Control top) {
		if (buttons == 2) {

		    FormData data = new FormData();
		    data.top = new FormAttachment(top);
		    data.bottom = new FormAttachment(100, 0);
		    data.left = new FormAttachment(0, 0);
		    data.right = new FormAttachment(50, 0);
		    return data;
			
		} else if (buttons == 3){
			FormData data = new FormData();
			data.top = new FormAttachment(top);
	    	data.bottom = new FormAttachment(100, 0);
	    	data.left = new FormAttachment(0, 0);
	    	data.right = new FormAttachment(33, 0);
	    	return data;
		} else return null;
	}
	
	public FormData getRightButtonData(int buttons, Control top) {
		if (buttons == 2) {

		    FormData data = new FormData();
		    data.top = new FormAttachment(top);
		    data.bottom = new FormAttachment(100, 0);
		    data.left = new FormAttachment(50, 0);
		    data.right = new FormAttachment(100, 0);
		    return data;
			
		} else if (buttons == 3){
			FormData data = new FormData();
	    	data.bottom = new FormAttachment(100, 0);
	    	data.left = new FormAttachment(66, 0);
	    	data.right = new FormAttachment(100, 0);
	    	return data;
		} else return null;
	}

	public FormData getCenterData(Control top) {
	    FormData data = new FormData();
		
		data.top = new FormAttachment(top, 5, SWT.BOTTOM);
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		data.bottom = new FormAttachment(80);
		
		return data;
	}

	public FormData getMiddleButtonData(Control left, Control right, Control top) {
	    FormData data = new FormData();
		
		data.top = new FormAttachment(top, 2);
	    data.bottom = new FormAttachment(100, 0);
	    data.right = new FormAttachment(right);
	    data.left = new FormAttachment(left);
	    
	    return data;
	}

	
}
