/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.modeller.controllers.BasicController;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;



public abstract class NameTextUndoableOperationDialog extends UndoableOperationDialog implements ModifyListener {

	private static final String COMMENT_STRING = "Comment (optional)";
	protected Text nameText;
	protected Text displayNameText;
	protected boolean nameReady = false;
	protected boolean propertyDialog = false;

	
	protected NameTextUndoableOperationDialog(Shell shell, IUndoContext undoContext, String title) {
		super(shell, undoContext, title);
	}

	protected void createNameText(Composite parent, String text, BasicController controller) {
		Composite composite = new Composite(parent, SWT.NULL);
	
		new Label(composite, SWT.NULL).setText(text);
		composite.setLayout(new GridLayout(1, true));
		if (composite.getParent().getLayout() instanceof GridLayout)
			composite.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		
		nameText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		if (controller != null) {
			nameText.setText(controller.getName());
			nameReady = true;
		}
		nameText.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		nameText.addModifyListener(this);
		
		new Label(composite, SWT.NULL).setText(COMMENT_STRING);
		
		displayNameText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		if (controller != null) {
			String dn = controller.getDisplayName();
			if (dn != null && !dn.equals(controller.getName()))
				displayNameText.setText(controller.getDisplayName());
		}
		
		displayNameText.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
	}
	
	public void modifyText(ModifyEvent e) {
		if (nameText.getText() != null && !nameText.getText().equals(""))
			nameReady = true;
		else nameReady = false;
		checkStatus();
	
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		composite.setLayout(new GridLayout(1, true));
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		return composite;
	}


}
