/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.controllers.PartDefinitionController;
import kumbang.modeller.ui.editor.operations.AddPartDefinitionOperation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;

public class PartDefinitionDialog extends DefinitionDialog implements ModifyListener {


	private class AddPossibleTypesDialog extends Dialog {

		
		private List components;
	
		
		protected AddPossibleTypesDialog(Shell parentShell) {
			super(parentShell);
		}
		
		protected Control createDialogArea(Composite parent) {
			Composite composite = new Composite(parent, SWT.NULL);	
			composite.setLayout(new GridLayout(1, true));
			composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			components = new List(composite, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
			components.setLayoutData(LayoutFactory.getInstance().getFillGridData());
			
			setItems();
			
			Button create = new Button(composite, SWT.NULL);
			create.setText("Create New " + (isFeature ? "Feature Type..." : "Component Type"));
			create.setLayoutData(LayoutFactory.getInstance().getButtonGridData());
		
			create.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
				
						new ComposableTypeDialog(getShell(), undoContext, modelController, PartDefinitionDialog.this, isFeature).open();
					
					AddPossibleTypesDialog.this.close();
				}});
			return composite;
		}
		
		private void setItems() {
			components.setItems(isFeature ? modelController.getFeatureTypes() : modelController.getComponentTypes());
			String [] excluded;
			if (partController == null)
				excluded = isFeature ? modelController.getExcludedFeatureTypes(typeController.getName()) : modelController.getExcludedComponentTypes(typeController.getName());
			else 
				excluded = partController.getPotentialTypes();	
			for (int i = 0; i < excluded.length; i++) 
				components.remove(excluded[i]);
			for (int i = 0; i < potentialTypes.getItemCount(); i++)
				components.remove(potentialTypes.getItem(i));
			
		}

		protected void okPressed() {
			String[] selected = components.getSelection();
			for (int i = 0; i < selected.length; i++)
				addPartName(selected[i]);
			updateButtonStatus();
			super.okPressed();
		}
		
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText("Add Possible Types");
			    
		}
	}
	
	private List potentialTypes;
	private Spinner lowerBound;
	private Spinner upperBound;

	private boolean isFeature;
	private boolean boundsOK = false;
	private Button noneButton;
	private Button sameButton;
	private Button diffButton;
	private String defaultType;
	private PartDefinitionController partController;
	private Button remove;
	private Button add;

	public PartDefinitionDialog(Shell shell, IUndoContext undoContext, ModelController modelController, ComposableTypeController controller) {
		super(shell, undoContext, controller, modelController, "Add " +(controller instanceof FeatureTypeController ? "Subfeature " : "Part " )+ "Definition");
		this.isFeature = controller instanceof FeatureTypeController;
	}

	public PartDefinitionDialog(Shell shell, IUndoContext undoContext, ModelController modelController, ComposableTypeController controller, PartDefinitionController partController) {
		super(shell, undoContext, controller, modelController, (partController.isFeature() ? "Subfeature " : "Part " )+ "Definition");
		this.partController = partController;
		this.isFeature = partController.isFeature();
		nameReady = true;
		updateStatus(Status.OK_STATUS);
	}
	
	
	public PartDefinitionDialog(IUndoContext undoContext, ModelController modelController, ComposableTypeController target, String defaultType) {
		this(null, undoContext, modelController, target);
		this.defaultType = defaultType;
	}


	protected Control createDialogArea(Composite parent) {
		Composite composite =(Composite)super.createDialogArea(parent);
		
		composite.setLayout(LayoutFactory.getInstance().getRowLayout());
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		createNameText(composite, isFeature ? "Subfeature Name:" : "Part Name:", partController);
		createPossibleTypes(composite);
		createCardinality(composite);
		createSimilarity(composite);
	
		return composite;
	}
	


	private void createPossibleTypes(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(LayoutFactory.getInstance().getFormLayout());
		
		Label label = new Label(composite, SWT.NULL);
		label.setText(isFeature ? "Possible Feature Types:" : "Possible Component Types:");
		
		
	//	composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		potentialTypes =  new List(composite, SWT.BORDER | SWT.MULTI | SWT.SCROLL_LINE | SWT.V_SCROLL);		
		potentialTypes.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				updateButtonStatus();
			}
			
		}); 
		
		//potentialTypes.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 2));
		if (defaultType != null) {
			potentialTypes.add(defaultType);
			nameText.setText(defaultType.toLowerCase());
			nameReady = true;
		}
		
		if (partController != null)
			potentialTypes.setItems(partController.getPotentialTypes());
		
		
		add = new Button(composite, SWT.NULL);
		add.setText("Add..");
		//add.setLayoutData(LayoutFactory.getInstance().getButtonGridData());
		add.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				new AddPossibleTypesDialog(getShell()).open();
				checkStatus();
			}
			
		}); 
		remove = new Button(composite, SWT.NULL);
		remove.setText("Remove");
		remove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				potentialTypes.remove(potentialTypes.getSelectionIndices());
				updateButtonStatus();
				checkStatus();
			}
		});
		//remove.setLayoutData(LayoutFactory.getInstance().getButtonGridData());	
	
		potentialTypes.setLayoutData(LayoutFactory.getInstance().getCenterData(label));

	    add.setLayoutData(LayoutFactory.getInstance().getLeftButtonData(2, potentialTypes));
	    remove.setLayoutData(LayoutFactory.getInstance().getRightButtonData(2, potentialTypes));
		
	    updateButtonStatus();
	}
	
	
	private void updateButtonStatus() {
		if (potentialTypes.getSelectionCount() > 0)
			remove.setEnabled(true);
		else remove.setEnabled(false);
		checkStatus();
	}
	
	private void createCardinality(Composite parent) {

		Group card = new Group(parent, SWT.NULL);
		card.setText("Cardinality");
		card.setLayout(new GridLayout(2, true));
		//card.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		new Label(card, SWT.NULL).setText(("Minimum:"))
		;
		lowerBound =  new Spinner(card, SWT.BORDER);
		lowerBound.addModifyListener(this);
		lowerBound.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkBounds();
				
			}
			}
		);
		lowerBound.setIncrement(1);
		lowerBound.setDigits(0);	
		lowerBound.setMinimum(0);
		lowerBound.setMaximum(Short.MAX_VALUE);
	
			
		
		
		
//		Composite max = new Composite(card, SWT.NULL);
//		max.setLayout(new GridLayout(2, false));
		new Label(card, SWT.NULL).setText(("Maximum:"));
		upperBound =  new Spinner(card, SWT.BORDER);
		upperBound.addModifyListener(this);
		upperBound.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkBounds();
				
				}
			}
		);
		upperBound.setIncrement(1);
		upperBound.setDigits(0);
		upperBound.setMinimum(1);
		
		if (partController != null) {
			upperBound.setSelection(partController.getUpperBound());
			if (partController.getLowerBound() > 0)
				lowerBound.setSelection(partController.getLowerBound());
		} else {
			lowerBound.setSelection(1);
			upperBound.setSelection(1);
		}
		upperBound.setMaximum(Short.MAX_VALUE);
	}
	
	private void createSimilarity(Composite parent) {
		Group sim = new Group(parent, SWT.NONE);
		sim.setText("Similarity");
		sim.setLayout(new GridLayout(1, true));
	   // sim.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
	
		
		noneButton = new Button(sim, SWT.RADIO);
		noneButton.setText("No further restrictions on chosen types");
		noneButton.setSelection(true);
		
		sameButton = new Button(sim, SWT.RADIO);
		sameButton.setText("All chosen types must be of the same type");
		
		diffButton = new Button(sim, SWT.RADIO);
		diffButton.setText("All chosen types must be of different types");
		
		if (partController != null) {
			switch(partController.getSimilarity()) {
			
			case PartDefinition.simNone:
				noneButton.setSelection(true);
				break;
			case PartDefinition.simSame:
				sameButton.setSelection(true);
				break;
			case PartDefinition.simDifferent:
				diffButton.setSelection(true);
				break;
			}
		}
				
	}
	


	 



	public void modifyText(ModifyEvent e) {
		
		if (e.getSource() == nameText) {
			super.modifyText(e);
			return;
		}
		
		checkBounds();
				
		
				
	
		
	}


	private void checkBounds() {
		boundsOK = lowerBound.getSelection() >= 0 && lowerBound.getSelection() <= upperBound.getSelection();
		
		if (!boundsOK) {
			status.setMessage("Minimum must be lower than maximum.");
			status.setThrowable(null);
		} else {
			status.setMessage("");
		}
		
		checkStatus();
		
	}


	protected void checkStatus() {
		if (boundsOK && nameReady && potentialTypes.getItemCount() > 0) 
			updateStatus(Status.OK_STATUS);
		else
			updateStatus(status);
		
	}

	protected void updateInfo() {
		partController.setName(nameText.getText());
		partController.setDisplayName(displayNameText.getText());
		partController.setLowerBound(lowerBound.getSelection());
		partController.setUpperBound(upperBound.getSelection());
		partController.setPossibleTypes(potentialTypes.getItems());
		partController.setSimilarity(getSimilarity());
		
	}

	protected void doNew() throws ExecutionException {

	
		doOperation(new AddPartDefinitionOperation(typeController, new Name(nameText.getText(), displayNameText.getText()), lowerBound.getSelection(), upperBound.getSelection(), getSimilarity(), potentialTypes.getItems()));
		
	}

	private int getSimilarity() {
		
		if (sameButton.getSelection())
			return PartDefinition.simSame;
		else if (diffButton.getSelection())
			return PartDefinition.simDifferent;
		else 
			return PartDefinition.simNone;
	}

	protected boolean isPropertyDialog() {
		return partController != null;
	}

	public void addPartName(String name) {
		potentialTypes.add(name);
		if (!nameReady) {
			nameText.setText(name.toLowerCase());
			nameReady = true;
		}
		
	}



	
}
