/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.modeller.controllers.AttributeDefinitionController;
import kumbang.modeller.controllers.AttributeValueTypeController;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ConnectionController;
import kumbang.modeller.controllers.ConstraintController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.InterfaceTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.controllers.PartDefinitionController;
import kumbang.modeller.ui.dialogs.constraints.ConstraintDialog;
import kumbang.modeller.util.Constants;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

public class PropertyDialogFactory  implements Constants{

	private static final PropertyDialogFactory instance = new PropertyDialogFactory();
	
	private PropertyDialogFactory() {
	
	}



	public static PropertyDialogFactory getInstance() {
		return instance;
	}



	public Dialog getPropertyDialog(Shell shell, BasicController controller, ModelController modelController, IUndoContext undoContext) {
		switch(controller.getType()) {
			
			case COMPONENT_TYPE:
				return new ComposableTypeDialog(shell, undoContext, modelController, (ComponentTypeController) controller, false);
			
			case FEATURE_TYPE:
				return new ComposableTypeDialog(shell, undoContext, modelController, (FeatureTypeController) controller, true);
			
			case ATTRIBUTE_VALUE_TYPE:			
				return new AttributeValueTypeDialog(shell, modelController,(AttributeValueTypeController) controller);
		
			case ATTRIBUTE_DEFINITION:
				return new AttributeDefinitionDialog(shell, undoContext, null, modelController, (AttributeDefinitionController)controller);
		
			case INTERFACE_TYPE:
				return new InterfaceTypeDialog(shell, undoContext, modelController, (InterfaceTypeController) controller);
		
			case INTERFACE_DEFINITION:
				return new InterfaceDefinitionDialog(shell, undoContext, null, modelController, (InterfaceDefinitionController) controller);
				
			case PART_DEFINITION:
				return new PartDefinitionDialog(shell, undoContext, modelController, null, (PartDefinitionController)controller);
	
			case CONNECTION:
				return new ConnectionDialog(shell, undoContext, (ConnectionController)controller, modelController);
			
			case CONSTRAINT:{
				return new ConstraintDialog(shell, undoContext, modelController, ((ConstraintController)controller));
			}
		}
		return null;
	}

}
