/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.modeller.KumbangModellerPlugin;
import kumbang.util.KumbangException;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.swt.widgets.Shell;


public abstract class UndoableOperationDialog extends ModifiableStatusDialog {
		
		protected IUndoContext undoContext;
		
		private String title;
		
		protected UndoableOperationDialog(Shell shell, IUndoContext undoContext, String title) {
			super(shell);
			this.undoContext = undoContext;
			this.title = title;
			
		}

		protected void doOperation(IUndoableOperation operation) throws ExecutionException {
			operation.addContext(undoContext);
	    	OperationHistoryFactory.getOperationHistory().execute(operation, null, null);
			
		}
		

		protected abstract void checkStatus();


		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText(title);
			    
		}
		
		 protected void okPressed() {
			 
			 if (!isPropertyDialog())
				try {
					doNew();
				} catch (Exception e) {
					status.setMessage(e.getMessage());
					status.setThrowable(e);
					updateStatus(status);
					return;
				}
			else
				try {
					updateInfo();
				} catch (Exception e) {
					status.setMessage(e.getMessage());
					status.setThrowable(e);
					updateStatus(status);
					return;
				}
			 
			 super.okPressed();
		 }

		protected abstract void updateInfo() throws Exception;

		protected abstract void doNew() throws ExecutionException, PropertyAlreadyDefinedException, Exception;

		protected abstract boolean isPropertyDialog();
		
}
