/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kumbang.core.model.constraint.AttributeExpression;
import kumbang.core.model.constraint.AttributeReference;
import kumbang.core.model.constraint.ComparisonExpression;
import kumbang.core.model.constraint.ConstantExpression;
import kumbang.core.model.constraint.IntegerExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;


public class AttributeExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Attribute Comparison";

	protected static final String VALUES_STRING = "VALUES:";
	protected static final String ATTRIBUTES_STRING = "ATTRIBUTES:";

	private Combo partCombo;	
	private Combo operatorCombo;
	private Combo valuePartCombo;

	private AttributeReference attributePartRef;
	
	private ComparisonExpression ce;

	private List<AttributeReference> refs;
	
	private List<AttributeReference> potentialValues = new ArrayList<AttributeReference>();
	
	protected int attributesOffset;

	protected boolean attributes;	
	

	public AttributeExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController, NAME);
	}

	public AttributeExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController, ComparisonExpression expression) {
		super(dialog, controller, parentShell, isImplementation, modelController, NAME);
		this.ce = expression;
		update  = true;
	}

	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
		
		new Label(composite, SWT.NULL).setText("Value of ");
		
		Composite attributeComposite = new Composite(composite, SWT.NULL);
		attributeComposite.setLayout(new GridLayout(1, true));
		attributeComposite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
	
		partCombo = new Combo(attributeComposite, SWT.READ_ONLY);
		refs = modelController.getAllAttributes(controller.getComposableType(), isImplementation);

		partCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		partCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				ce = null;
				updateStatus(status);
				
				
				attributePartRef = refs.get(partCombo.getSelectionIndex());
				
				updateValues();
				
			}
		});
		
		new Label(composite, SWT.NULL).setText("is");
		
		Composite opsComposite = new Composite(composite, SWT.NULL);
		opsComposite.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		opsComposite.setLayout(new GridLayout());
		
		operatorCombo = new Combo(opsComposite, SWT.READ_ONLY);
		operatorCombo.setItems(ComparisonExpression.ops);
		operatorCombo.setItem(0, "=");
		operatorCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkStatus();
			}
		});
		
		new Label(composite, SWT.NULL).setText("compared to");
		
		Composite valueComposite = new Composite(composite, SWT.NULL);
		valueComposite.setLayout(new GridLayout(1, true));
		valueComposite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		valuePartCombo = new Combo(valueComposite, SWT.READ_ONLY);
		

		valuePartCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		valuePartCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkStatus();
			}
		
		});

		
		for (Iterator<AttributeReference> it = refs.iterator(); it.hasNext();) {
		
			partCombo.add(it.next().getModelString(isImplementation));
		//	valuePartCombo.add(next.toString());
	
		
		}

		
		if (ce != null) {
			setExpression();
			
			
		}
		return composite;
	}

	protected void setExpression() {
		
		if (ce == null)
			return;
		
		attributePartRef = ((AttributeExpression)ce.getLeft()).getAttributeReference();
		for (int i = 0; i < partCombo.getItemCount(); i++){
			if (attributePartRef.getModelString(isImplementation).equals(partCombo.getItem(i))) {
				attributePartRef = refs.get(i);
				partCombo.select(i);
				break;
			}
		}
		
		
		for (int i = 0; i < operatorCombo.getItemCount(); i++) {
			if (operatorCombo.getItem(i).equals(ce.getOperator().replace("==", "="))) {
				operatorCombo.select(i);
				break;
			}
		}
		updateValues();
		
		String valueString = "";
		if (ce.getRight() instanceof AttributeExpression)
			valueString = ((AttributeExpression)ce.getRight()).getAttributeReference().getModelString(isImplementation);
		else if (ce.getRight() instanceof ConstantExpression)
			valueString = ((ConstantExpression)ce.getRight()).getModelString(isImplementation);
		
		for (int i = 0; i < valuePartCombo.getItemCount(); i++){
			if (valueString.equals(valuePartCombo.getItem(i))) {
				valuePartCombo.select(i);
				break;
			}
		}
		checkStatus();
		
	}

	protected void updateValues() {
		valuePartCombo.removeAll();
		
		valuePartCombo.add(VALUES_STRING);
		if (attributePartRef.getType().getValues() != null)
			for (Iterator<Name> it = attributePartRef.getType().getValues(); it.hasNext();) {			
				valuePartCombo.add(it.next().getName());
			}	
		
		attributesOffset = valuePartCombo.getItemCount();

		potentialValues.clear();
		
		for (Iterator<AttributeReference> it = refs.iterator(); it.hasNext();) {	
			AttributeReference ref = it.next();
			if (ref != attributePartRef && ref.getType().getName().equals(attributePartRef.getType().getName())) {
				valuePartCombo.add(ref.getModelString(isImplementation));
				potentialValues.add(ref);
			}
	
		
		}
	
		
		if (valuePartCombo.getItemCount() > attributesOffset) {
			valuePartCombo.add(ATTRIBUTES_STRING, attributesOffset++);
			attributes = true;
		} else
			attributes = false;
		
		
		
		
	}

	protected void checkStatus() {	
		
		if (operatorCombo.getSelectionIndex() < 0) {
			return;
		}
		
		
		if ((attributes && valuePartCombo.getSelectionIndex() == attributesOffset - 1) || valuePartCombo.getSelectionIndex() <= 0)  {
			updateStatus(status);
			return;
		}

		AttributeExpression left = new AttributeExpression(attributePartRef);

		IntegerExpression right;
		if (valuePartCombo.getSelectionIndex() < attributesOffset)
			right = new ConstantExpression(valuePartCombo.getItem(valuePartCombo.getSelectionIndex()));
		else
			right = new AttributeExpression(potentialValues.get(valuePartCombo.getSelectionIndex() - attributesOffset));
		
		ce = new ComparisonExpression(left, right, operatorCombo.getSelectionIndex());
		updateStatus(Status.OK_STATUS);

	
	}

	public void okPressed() {
		if (ce != null) {
			super.okPressed();
			dialog.addCondition(ce, update);
		}
	}
	
	
	
	
}

