/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.model.constraint.AttributeExpression;
import kumbang.core.model.constraint.AttributeReference;
import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.constraint.CardinalityExpression;
import kumbang.core.model.constraint.ComparisonExpression;
import kumbang.core.model.constraint.ConstantExpression;
import kumbang.core.model.constraint.Expression;
import kumbang.core.model.constraint.IntegerExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.Name;
import kumbang.core.model.type.PartDefinition;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;

public class CardinalityExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Cardinality";
	private Combo partCombo;
	protected PartDefinition cardinalityPartDef;
;
	private Combo operatorCombo;
	private Combo valuePartCombo;
	private ComparisonExpression ce;
	private List<StructureReference> refs;
	private Button value;
	private Button number;
	private Spinner card;
	private Combo valueValueCombo;
	protected StructureReference cardinalityRef;
	protected StructureReference valueRef;

	
	public CardinalityExpressionDialog(ConstraintDialog dialog,
			ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController);		
	}

	public CardinalityExpressionDialog(ConstraintDialog dialog, ComposableTypeController composableController, Shell shell, boolean isImplementation, ModelController modelController, ComparisonExpression expression) {
		super(dialog, composableController, shell, isImplementation, modelController, NAME);

		this.ce = expression;
		update = true;
	}

	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
		
		
		new Label(composite, SWT.NULL).setText("Cardinality of ");
		
		Composite cardinalityComposite = new Composite(composite, SWT.NULL);
		cardinalityComposite.setLayout(new GridLayout(1, true));
		cardinalityComposite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		partCombo = new Combo(cardinalityComposite, SWT.READ_ONLY);
		
		refs = modelController.getAllParts(controller.getComposableType());
		if(isImplementation) 
			refs.addAll(modelController.getAllComponentParts());
		
		partCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		partCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				int index = partCombo.getSelectionIndex();
				cardinalityRef  = refs.get(index); 
				cardinalityPartDef = modelController.getPartDefinition(cardinalityRef);
				card.setMinimum(cardinalityPartDef.getLowerBound());
				card.setMaximum(cardinalityPartDef.getUpperBound());
				
			}
		});

		new Label(composite, SWT.NULL).setText("is");
		
		Composite opsComposite = new Composite(composite, SWT.NULL);
		opsComposite.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		opsComposite.setLayout(new GridLayout());
		
		operatorCombo = new Combo(opsComposite, SWT.READ_ONLY);
		operatorCombo.setItems(ComparisonExpression.ops);
		operatorCombo.setItem(0, "=");
	
		operatorCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkStatus();
			}
		});
		
		new Label(composite, SWT.NULL).setText("compared to");
		
		
		Composite valueComposite = new Composite(composite, SWT.NULL);
		valueComposite.setLayout(new FillLayout(SWT.VERTICAL));
		valueComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
		 
		
		Group valueGroup = new Group(valueComposite, SWT.NULL);
		valueGroup.setLayout(new GridLayout());
		//valueGroup.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		value = new Button(valueGroup, SWT.RADIO);
		value.setText("Compare to an attribute");
		value.setSelection(true);
		value.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				number.setSelection(false);
			}
		});
		
		valuePartCombo = new Combo(valueGroup, SWT.NULL);
		valuePartCombo.add("(this)");
		valuePartCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		valuePartCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				updateValues();
			}
		});	
		valueValueCombo = new Combo(valueGroup, SWT.NULL);
		valueValueCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		valueValueCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				checkStatus();
				
			}});
		
		Group spinnerGroup = new Group(valueComposite, SWT.NULL);
		spinnerGroup.setLayout(new GridLayout());
		number = new Button(spinnerGroup, SWT.RADIO);
		number.setText("Compare to a number value");

		number.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				value.setSelection(false);
				checkStatus();
			}
		});
		card = new Spinner(spinnerGroup, SWT.BORDER);
		card.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				value.setSelection(false);
				number.setSelection(true);
				checkStatus();
			}
		});
		for (Iterator<StructureReference> it = refs.iterator(); it.hasNext();) {
			StructureReference next = it.next();
			partCombo.add(next.getModelString(isImplementation));
			valuePartCombo.add(next.getModelString(isImplementation));
		
		}
		
		if (ce != null)
			setExpression();
		
		return composite;
	}

	protected void updateValues() {
		if (valuePartCombo.getSelectionIndex() == 0) {
			valueValueCombo.setItems(controller.getAttributes());
			return;
		}		
		
		int index = valuePartCombo.getSelectionIndex() - 1;
	
		valueRef = refs.get(index);
		valueValueCombo.removeAll();
		
		List<String> attributes = new LinkedList<String>();
	
		for (Iterator<ComposableType> i = valueRef.possibleTypes(); i.hasNext();) {
			Iterator<AttributeDefinition> it = i.next().getAttributes();
			while(it.hasNext())  {
				String name = it.next().getName();
				if (!attributes.contains(name)) {
					valueValueCombo.add(name);
					attributes.add(name);
				
			
				}
			}
		}
		
	}

	protected void checkStatus() {
		CardinalityExpression left  = new CardinalityExpression(cardinalityRef);
		IntegerExpression right;
	
		if (number.getSelection()) 		
			right = new ConstantExpression(card.getSelection());
		else if (valueValueCombo.getSelectionIndex() == 0) 
			right = new AttributeExpression(new AttributeReference(controller.getComposableType(), valueValueCombo.getItem(valueValueCombo.getSelectionIndex())));
		else 
			right = new AttributeExpression(new AttributeReference(valueRef,valueValueCombo.getItem(valueValueCombo.getSelectionIndex())));
		
		ce = new ComparisonExpression(left, right, operatorCombo.getSelectionIndex());	
		updateStatus(Status.OK_STATUS);
		
	}
	
	public void okPressed() {
		if (ce != null) {
			super.okPressed();
			dialog.addCondition(ce, update);
		}
	}

	@Override
	protected void setExpression() {
		if (ce == null)
			return;
		
		cardinalityRef = ((CardinalityExpression)ce.getLeft()).getReference();
		for (int i = 0; i < partCombo.getItemCount(); i++){
			if (cardinalityRef.getModelString(isImplementation).equals(partCombo.getItem(i))) {
				cardinalityRef = refs.get(i);
				partCombo.select(i);
				break;
			}
		}
		
		
		for (int i = 0; i < operatorCombo.getItemCount(); i++) {
			if (operatorCombo.getItem(i).equals(ce.getOperator().replace("==", "="))) {
				operatorCombo.select(i);
				break;
			}
		}
		
		cardinalityPartDef = modelController.getPartDefinition(cardinalityRef);
		card.setMinimum(cardinalityPartDef.getLowerBound());
		card.setMaximum(cardinalityPartDef.getUpperBound());
		
		if (ce.getRight() instanceof ConstantExpression) {
			card.setSelection(((ConstantExpression)ce.getRight()).getValue());
			number.setSelection(true);
			value.setSelection(false);
		} else if (ce.getRight() instanceof AttributeExpression) {
			number.setSelection(false);
			value.setSelection(true);
			
			
			AttributeExpression ae = (AttributeExpression) ce.getRight();
			valueRef = ae.getStructureReference();
			if (ae.getStructureReference() == null)
				valuePartCombo.select(0);
			else {
				for (int i = 0; i < valuePartCombo.getItemCount(); i++) {
					if (valueRef.getModelString(isImplementation).equals(valuePartCombo.getItem(i))) {
						valueRef = refs.get(i);
						valuePartCombo.select(i);
						break;
					}
				}
			}
	
			
			updateValues();
			
			AttributeReference attref = ((AttributeExpression)ce.getRight()).getAttributeReference();
			for (int i = 0; i < valueValueCombo.getItemCount(); i++){

				if (attref.getModelString(isImplementation).equals(valueValueCombo.getItem(i))) {
					valueValueCombo.select(i);
					break;
				}
			}
			
		
		
		}
		checkStatus();

	}
}
