/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import kumbang.core.model.constraint.ConnectedExpression;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class ChooseExpressionTypeDialog extends ConstraintExpressionDialog {
	
	private static final String LABEL = "Choose expression type:";

	private Combo types;
	
	private String[] expressionTypes = new String[]{AttributeExpressionDialog.NAME, PresentExpressionDialog.NAME, CardinalityExpressionDialog.NAME, InstanceOfExpressionDialog.NAME, HasInstancesExpressionDialog.NAME, HasPartOfTypeExpressionDialog.NAME}; //"Cardinality", "Connected", "Has instances", "Has part of type", "Instance of", "Present"};
	
	protected ChooseExpressionTypeDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell shell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, shell, isImplementation, modelController);
		setTitle("Choose Expression Type");
	}
	
	public Control createDialogArea(Composite parent) {
	
		
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		new Label(composite, SWT.NULL).setText(LABEL);
		types = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);		
		types.setItems(expressionTypes);
		if (isImplementation || controller instanceof ComponentTypeController)
			types.add(ConnectedExpressionDialog.NAME);
		types.setLayoutData(LayoutFactory.getInstance().getFillGridData());	
		types.addSelectionListener(new SelectionAdapter(){
		
			public void widgetSelected(SelectionEvent e) {

				updateStatus(Status.OK_STATUS);

			}

		});
		
		return composite;	
	}
	
	public void okPressed() {
		String selection = types.getItem(types.getSelectionIndex());
		Dialog d = null;
		if (AttributeExpressionDialog.NAME.equals(selection)) {		
			d = new AttributeExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);		
		} else if (PresentExpressionDialog.NAME.equals(selection)) {
			d = new PresentExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		} else if (CardinalityExpressionDialog.NAME.equals(selection)) {
			d = new CardinalityExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		} else if (InstanceOfExpressionDialog.NAME.equals(selection)) {
			d = new InstanceOfExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		} else if (HasInstancesExpressionDialog.NAME.equals(selection)) {
			d = new HasInstancesExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		} else if (HasPartOfTypeExpressionDialog.NAME.equals(selection)) {
			d = new HasPartOfTypeExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		} else if (ConnectedExpressionDialog.NAME.equals(selection)) {
			d = new ConnectedExpressionDialog(dialog, controller, getParentShell(), isImplementation, modelController);
		}

		if (d != null) {
			super.okPressed();
			d.open();
		}
	}

	@Override
	protected void setExpression() {
		// TODO Auto-generated method stub
		
	}
	
	

}
