/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.ArrayList;
import java.util.List;

import kumbang.core.model.constraint.ConnectedExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class ConnectedExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Connected";
	
	private Combo firstInterfaceCombo;
	private List<StructureReference> firstIntRefs;
	private List<StructureReference> secondIntRefs;

	private Combo secondInterfaceCombo;

	private Button button;

	private ConnectedExpression ce;

	protected ConnectedExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController);
		
	}

	public ConnectedExpressionDialog(ConstraintDialog dialog, ComposableTypeController composableController, Shell shell, boolean isImplementation, ModelController modelController, ConnectedExpression expression) {
		this(dialog, composableController, shell, isImplementation, modelController);
		update = true;
		ce = expression;
	}

	@Override
	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
		
		new Label(composite, SWT.NULL).setText("Select the connected interface:");
		
		firstInterfaceCombo = new Combo(parent, SWT.READ_ONLY);
		firstInterfaceCombo.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
						
				if (button.getSelection()) {
					updateInterfaces();
					updateStatus(status);
				}
				else  {
					secondIntRefs = null;
					updateStatus(Status.OK_STATUS);
				}
				
			}}); 
		firstInterfaceCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		
		firstIntRefs = modelController.getAllInterfaces(controller.getComposableType());
	
		
		for (StructureReference ref : firstIntRefs)
			firstInterfaceCombo.add(ref.getModelString(isImplementation));
		
		button = new Button(parent, SWT.CHECK);
		button.setText("Connect to specific interface");
		button.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
				
				secondInterfaceCombo.setEnabled(button.getSelection()); 
				if (button.getSelection() && firstInterfaceCombo.getSelectionIndex() >=0) {
					updateStatus(status);
					updateInterfaces();
				}
				else  {
					secondIntRefs = null;
					updateStatus(Status.OK_STATUS);
				}
			}}); 
		
		secondInterfaceCombo = new Combo(parent, SWT.READ_ONLY);
		secondInterfaceCombo.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				updateStatus(Status.OK_STATUS);
				
			}
			
		});
		secondInterfaceCombo.setEnabled(false);
		secondInterfaceCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		return composite;
	}

	protected void updateInterfaces() {
		List<StructureReference> allInterfaces = modelController.getAllInterfaces(modelController.getComponentRoot());
		secondIntRefs = new ArrayList<StructureReference>();
		StructureReference firstSelected = firstIntRefs.get(firstInterfaceCombo.getSelectionIndex());
		for (StructureReference ref : allInterfaces)
			if (canConnect(ref, firstSelected))
				secondIntRefs.add(ref);
		
		for (StructureReference ref : secondIntRefs)
			secondInterfaceCombo.add(ref.getModelString(isImplementation));
		
	}
	
	@SuppressWarnings("unchecked")
	private boolean canConnect(StructureReference source, StructureReference target) {
				
		if (source == target) {
			return false;
		}	
		
		InterfaceDefinition sourceInterface = modelController.getInterfaceDefinition(source);
		InterfaceDefinition targetInterface = modelController.getInterfaceDefinition(target);
	
		if (sourceInterface.equals(targetInterface) && source.equals(target) )
			return false;
		
		
		ComponentTypeController componentController;
		
		if (controller instanceof ComponentTypeController)
			componentController = (ComponentTypeController)controller;
		else componentController = modelController.getComponentRootController();
		
		if (sourceInterface == null)
			sourceInterface = componentController.getInterface(source.getInterfaceName());
		if (targetInterface == null)
			targetInterface = componentController.getInterface(target.getInterfaceName());
		
		if (sourceInterface != null && targetInterface!= null) {
		
				
			List<InterfaceType> sourceInterfaceTypeList = sourceInterface.getPossibleTypesList();
			List<InterfaceType> targetInterfaceTypeList = targetInterface.getPossibleTypesList();
			for (int i = 0; i < sourceInterfaceTypeList.size() && i < targetInterfaceTypeList.size(); i++){		
				if (sourceInterfaceTypeList.get(i).isDisjointFrom((targetInterfaceTypeList.get(i)))) {
					return false;
				}
			}		
		}	
		
		if (ce != null)
			setExpression();
		
		return true;
	
	}
	
	
	public void okPressed() {
		StructureReference from = firstIntRefs.get(firstInterfaceCombo.getSelectionIndex());
		
		if (button.getSelection()) 
			dialog.addCondition(new ConnectedExpression(from, secondIntRefs.get(secondInterfaceCombo.getSelectionIndex())), update);
		else 
			dialog.addCondition(new ConnectedExpression(from), update);
		
		super.okPressed();
	}

	@Override
	protected void setExpression() {
		for (int i = 0; i < firstInterfaceCombo.getItemCount(); i++)
			if (ce.getFromReference().getModelString(isImplementation).equals(firstInterfaceCombo.getItem(i))) {
				firstInterfaceCombo.select(i);
			}
		updateStatus(Status.OK_STATUS);

		
		if (ce.getToReference() == null) {
			button.setSelection(false);
			return;
		}
		
		updateInterfaces();
		button.setSelection(true);
		secondInterfaceCombo.setEnabled(true);
		
		for (int i = 0; i < secondInterfaceCombo.getItemCount(); i++)
			if (ce.getToReference().getModelString(isImplementation).equals(secondInterfaceCombo.getItem(i))) {
				secondInterfaceCombo.select(i);
			}
		
	}

}
