/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import kumbang.core.model.constraint.HasInstancesExpression;
import kumbang.core.model.type.ElementaryType;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class HasInstancesExpressionDialog extends ConstraintExpressionDialog {

	
	public static final String NAME = "Has Instances";
	private static final String NEGATION = "Does not have instances";
	private Combo partCombo;
	private Button not;
	private String[] attributes;
	private String[] features;
	private String[] interfaces;
	private String[] components;
	private int featuresEndIndex;
	private int componentsEndIndex;
	private int attributesEndIndex;
	private ElementaryType type;
	private HasInstancesExpression hie;

	protected HasInstancesExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController, NAME);
		
	}

	public HasInstancesExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell shell, boolean isImplementation, ModelController modelController, HasInstancesExpression expression) {
		this(dialog, controller, shell, isImplementation, modelController);
		update = true;
		hie = expression;
	}

	@Override
	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
		
		partCombo = new Combo(composite, SWT.READ_ONLY);
		partCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				int index = partCombo.getSelectionIndex();
			
				
				if (featuresEndIndex != 0 && index < featuresEndIndex) {
					type = modelController.getFeatureType(partCombo.getItem(index));
				} else if (componentsEndIndex != 0 && index < componentsEndIndex) {
					type = modelController.getComposableType(partCombo.getItem(index));
				} else if (attributesEndIndex != 0 && index < attributesEndIndex) {
					type = modelController.getAttributeType(partCombo.getItem(index));
				} else {
					type = modelController.getInterfaceType(partCombo.getItem(index));
				}
					
				if (type == null)
					updateStatus(status);
				else			
					updateStatus(Status.OK_STATUS);
					
			}
			
		});
		partCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		
		not = new Button(composite, SWT.CHECK);
		not.setText(NEGATION);
		
		attributes = modelController.getAttributeTypes();
		
		if (controller instanceof FeatureTypeController) 
			features = modelController.getFeatureTypes();
		else 
			features = new String[0];
		
		if (controller instanceof ComponentTypeController || isImplementation) {
			components = modelController.getComponentTypes();
			interfaces = modelController.getInterfaces();
		} else {
			components = new String[0];
			interfaces = new String[0];
		}
		
		if (features.length > 0) {
			partCombo.add("FEATURE TYPES:");
			for(String s : features)
				partCombo.add(s);
			featuresEndIndex = partCombo.getItemCount();
		}
		
		if (components.length > 0) {	
			partCombo.add("COMPONENT TYPES:");				
			for(String s : components)
				partCombo.add(s);
			componentsEndIndex = partCombo.getItemCount();
			
		}
		
		if (attributes.length > 0) {	
			partCombo.add("ATTRIBUTE TYPES:");
			for(String s : attributes)
				partCombo.add(s);
			attributesEndIndex = partCombo.getItemCount();
		}
		if (interfaces.length > 0) {			
			partCombo.add("INTERFACE TYPES:");
			for(String s : interfaces)
				partCombo.add(s);
		}	
		
		if (hie != null)
			setExpression();
		return composite;
	}

	public void okPressed() {
		
		dialog.addCondition(new HasInstancesExpression(type), update);
		
		super.okPressed();
	}

	@Override
	protected void setExpression() {
		
		type = hie.getComparedType();
		
		for (int i = 0; i < partCombo.getItemCount(); i++) {
			if (partCombo.equals(type.getName())) {
				partCombo.select(i);
				updateStatus(Status.OK_STATUS);
				return;
			}
		}
	}
	
}
