/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.List;

import kumbang.core.model.constraint.HasPartOfTypeExpression;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class HasPartOfTypeExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Has Part Of Type";
	private Combo exCombo;
	private List<HasPartOfTypeExpression> exs;
	private HasPartOfTypeExpression hpote;


	protected HasPartOfTypeExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController);
		setTitle("Has Part Of Type Expression");
	}

	public HasPartOfTypeExpressionDialog(ConstraintDialog dialog, ComposableTypeController composableController, Shell shell, boolean isImplementation, ModelController modelController, HasPartOfTypeExpression expression) {
		this (dialog, composableController, shell, isImplementation, modelController);
		update = true;
		hpote = expression;
	}

	@Override
	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());		
		
		new Label(composite, SWT.NULL).setText("Choose the part and the type");
		
		exCombo = new Combo(composite, SWT.NULL);	
		exs = modelController.getPotentialPartTypes(controller.getComposableType(), isImplementation);
		

		exCombo.addSelectionListener(new SelectionAdapter(){
		
			
			public void widgetSelected(SelectionEvent e) {

				updateStatus(Status.OK_STATUS);

			}

		});
		
		for (HasPartOfTypeExpression ex : exs)
			exCombo.add(ex.getModelString(isImplementation));
		
		if (hpote != null)
			setExpression();
		return composite;
	}

	
	public void okPressed() {
		dialog.addCondition(exs.get(exCombo.getSelectionIndex()), update);
		super.okPressed();
	}

	@Override
	protected void setExpression() {
		for (int i = 0; i < exCombo.getItemCount(); i++ )
			if (exCombo.getItem(i).equals(hpote.getModelString(isImplementation))) {
				exCombo.select(i);
				updateStatus(Status.OK_STATUS);
				return;
			}
		
	}
}
