/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.Iterator;
import java.util.List;

import kumbang.core.model.constraint.InstanceOfExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ElementaryType;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class InstanceOfExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Instance of";
	private Combo partsCombo;
	private Combo typeCombo;
	protected StructureReference ref;
	private List<StructureReference> refs;
	private Button not;
	private InstanceOfExpression ioe;
	
	public InstanceOfExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController);	
	}


	public InstanceOfExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell shell, boolean isImplementation, ModelController modelController, InstanceOfExpression expression) {
		this(dialog, controller, shell, isImplementation, modelController);
		update = true;
		ioe = expression;
	}


	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());
	
		new Label(composite, SWT.NULL).setText("Instance of");
	
		refs = modelController.getAllParts(controller.getComposableType());
		
		if (isImplementation)
			for (StructureReference ref : modelController.getAllComponentParts())
			refs.add(ref);
		
		partsCombo = new Combo(composite, SWT.READ_ONLY);
		partsCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		for (Iterator<StructureReference> it = refs.iterator(); it.hasNext();)
			partsCombo.add(it.next().getModelString(isImplementation));
		
		partsCombo.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
				ref = refs.get(partsCombo.getSelectionIndex());
				updateTypes();
				
			}});
		new Label(composite, SWT.NULL).setText("is");
		not = new Button(composite, SWT.CHECK);
		not.setText("not");
		new Label(composite, SWT.NULL).setText("of type");
		typeCombo = new Combo(composite, SWT.READ_ONLY);
		typeCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		typeCombo.addSelectionListener(new SelectionAdapter(){

			public void widgetSelected(SelectionEvent e) {
				updateStatus(Status.OK_STATUS);
			}});
		
		if (ioe != null)
			setExpression();
		return composite;
	}

	
	protected void updateTypes() {
		typeCombo.removeAll();
		updateStatus(status);
		
		for (Iterator<ElementaryType> it = ref.possibleTypes(); it.hasNext();)
			typeCombo.add(it.next().getName());
		
	}


	public void okPressed() {
		
		ioe = new InstanceOfExpression(ref, modelController.getComposableType(typeCombo.getItem(typeCombo.getSelectionIndex())));
		ioe.setNegated(not.getSelection());
		dialog.addCondition(ioe, update);
		super.okPressed();
	}


	@Override
	protected void setExpression() {
		if (ioe == null)
			return;
		
		ref = ioe.getReference();
		
		for (int i = 0; i < partsCombo.getItemCount(); i++) {
			if (partsCombo.getItem(i).equals(ref.getModelString(isImplementation))) {
				partsCombo.select(i);
				break;
			}
		}
		updateTypes();
		
		if (ioe.getType() != null) {
		
			for (int i = 0; i < typeCombo.getItemCount(); i++)
				if (ioe.getType().getName().equals(typeCombo.getItem(i))) {
					typeCombo.select(i);
					break;
				}
				
		}
		not.setSelection(ioe.isNegated());
		updateStatus(Status.OK_STATUS);
		
	}
	
}
