/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dialogs.constraints;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kumbang.core.model.constraint.BooleanExpression;
import kumbang.core.model.constraint.PresentExpression;
import kumbang.core.model.constraint.StructureReference;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

public class PresentExpressionDialog extends ConstraintExpressionDialog {

	public static final String NAME = "Present";

	private static final String NEGATION = "Is not present";
	
	private Button not;
	private Combo partCombo;
	
	private List<StructureReference> refs;

	private PresentExpression pe;


	protected PresentExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell parentShell, boolean isImplementation, ModelController modelController) {
		super(dialog, controller, parentShell, isImplementation, modelController);
		setTitle(NAME);
	}

	public PresentExpressionDialog(ConstraintDialog dialog, ComposableTypeController controller, Shell shell, boolean isImplementation, ModelController modelController, PresentExpression expression) {
		this(dialog, controller,shell, isImplementation, modelController);
		update = true;
		this.pe = expression;
	}

	public Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		composite.setLayout(new GridLayout());

	
		
		partCombo = new Combo(composite, SWT.READ_ONLY);
		partCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				updateStatus(Status.OK_STATUS);
				
			}
			
		});
		partCombo.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		
		not = new Button(composite, SWT.CHECK);
		not.setText(NEGATION);
		
		refs = modelController.getPossiblePresentTypes(controller.getComposableType(), false);
		if (isImplementation)
			refs.addAll(modelController.getPossiblePresentTypes(controller.getComposableType(), true));
		

		for (Iterator<StructureReference> it = refs.iterator(); it.hasNext();) {
		
			partCombo.add(it.next().getModelString(isImplementation));	
		}
		
		if (pe != null)
			setExpression();
		return composite;
	}

	

	public void okPressed() {

		int index = partCombo.getSelectionIndex();
		pe = new PresentExpression(refs.get(index));
		pe.setNegated(not.getSelection());		
		dialog.addCondition(pe, update);

		super.okPressed();
	}

	@Override
	protected void setExpression() {
		if (pe == null)
			return;
		
		StructureReference ref = pe.getReference();
		
		for (int i = 0; i < partCombo.getItemCount(); i++) {
			if (partCombo.getItem(i).equals(ref.getModelString(isImplementation))) {
				partCombo.select(i);
				break;
			}
		}
		
		not.setSelection(pe.isNegated());
		
		updateStatus(Status.OK_STATUS);
	}
}
