/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dnd;

import kumbang.modeller.controllers.AttributeDefinitionController;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.PartDefinitionController;
import kumbang.modeller.ui.viewers.FeatureTreeViewer;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;

public class FeatureViewDragSourceListener implements DragSourceListener {

	private FeatureTreeViewer viewer;
	
	
	public FeatureViewDragSourceListener(FeatureTreeViewer viewer) {
		this.viewer = viewer;
	}
	
	public void dragStart(DragSourceEvent event) {
		
		if (viewer.getSelection().isEmpty()) {
			event.doit = false;
			return;
		}
		event.detail = DND.DROP_MOVE;
		event.doit = viewer.getSelectedController() instanceof PartDefinitionController || viewer.getSelectedController() instanceof AttributeDefinitionController;
		
	}

	public void dragSetData(DragSourceEvent event) {
		
		if (viewer.getSelectedController() instanceof AttributeDefinitionController) {
			AttributeDefinitionController controller = (AttributeDefinitionController)viewer.getSelectedController();
			event.data = new DefinitionTransferData(controller.getName(), controller.getParentName(), controller.getType(), controller.getParentType());
		} else if (viewer.getSelectedController() instanceof PartDefinitionController) {
			PartDefinitionController controller = (PartDefinitionController)viewer.getSelectedController();
			event.data = new DefinitionTransferData(controller.getName(), controller.getParentName(), controller.getType(), controller.getParentType());
		}
		
	}

	public void dragFinished(DragSourceEvent event) {
		 viewer.refresh();		
	}


}
