/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dnd;

import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.util.Constants;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

public abstract class KumbangModellerDropAdapter extends ViewerDropAdapter implements Constants {

	protected ModelController modelController;
	protected IUndoContext undoContext;

	public KumbangModellerDropAdapter(TreeViewer viewer, ModelController controller, IUndoContext undoContext) {
		super(viewer);
		this.modelController = controller;
		this.undoContext = undoContext;

	}
	
    protected boolean doOperation(IUndoableOperation operation) {   
    	operation.addContext(undoContext);
    	try {
    		OperationHistoryFactory.getOperationHistory().execute(operation, null, null);
    	} catch (ExecutionException e) {
    		return false;
    	}	
    	return true; 
    }
    
	
    public void dragEnter(DropTargetEvent event) {
    	// A fix to get drag'n'drop working. Should not be necessary.
        event.detail = DND.DROP_COPY;
        super.dragEnter(event);
     
    }





}
