/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.dnd;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

public abstract class KumbangTransfer extends ByteArrayTransfer {
	
    protected void javaToNative(Object data, TransferData transferData) {
    	
        if (data == null)
            return;
        
        if (data instanceof TypeTransferData) {
    	
    	
        	TypeTransferData realData = (TypeTransferData) data;
    
        	try {
        		ByteArrayOutputStream out = new ByteArrayOutputStream();
        		DataOutputStream dataOut = new DataOutputStream(out);
        		dataOut.writeUTF(realData.getName());
        		dataOut.writeInt(realData.getType());
        		dataOut.close();
        		super.javaToNative(out.toByteArray(), transferData);
        	} catch (IOException e) {
        		e.printStackTrace();
        	}
        } else if (data instanceof DefinitionTransferData) {
        	DefinitionTransferData realData = (DefinitionTransferData) data;
            
        	try {
        		ByteArrayOutputStream out = new ByteArrayOutputStream();
        		DataOutputStream dataOut = new DataOutputStream(out);
        		dataOut.writeUTF(realData.getName());
        		dataOut.writeInt(realData.getType());
        		dataOut.writeUTF(realData.getParentName());
        		dataOut.writeInt(realData.getParentType());
        		dataOut.close();
        		super.javaToNative(out.toByteArray(), transferData);
        	} catch (IOException e) {
        		e.printStackTrace();
        	}
        }
    }

    protected Object nativeToJava(TransferData transferData) {
        try {
            byte[] bytes = (byte[]) super.nativeToJava(transferData);
            ByteArrayInputStream in = new ByteArrayInputStream(bytes);
            DataInputStream dataIn = new DataInputStream(in);
            String name = dataIn.readUTF();
            int type = dataIn.readInt();
            
            if (dataIn.available() == 0)
            	return new TypeTransferData(name, type);
            
            String parentName = dataIn.readUTF();
            int parentType = dataIn.readInt();
            
            return new DefinitionTransferData(name, parentName, type, parentType);
            
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}
