/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.editor.operations;

import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

public class AddInterfaceDefinitionOperation extends AddDefinitionOperation {

	private boolean isGrounded;
	private boolean isOptional;
	private boolean isProvides;

	private Name definitionName;
	private String[] types;

	public AddInterfaceDefinitionOperation(ComposableTypeController controller, Name name, String[] types, boolean grounded, boolean optional, boolean provides) {
		super("Add Interface", controller);
		this.types = types;
		this.definitionName = name;
		this.isGrounded = grounded;
		this.isOptional = optional;
		this.isProvides = provides;
	}

	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (controller.getType() == BasicController.COMPONENT_TYPE) {
			((ComponentTypeController)controller).addInterfaceDefinition(definitionName,  types, isGrounded, isOptional, isProvides);
			return Status.OK_STATUS;
		}
		return Status.CANCEL_STATUS;
	}

	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (controller.getType() == BasicController.COMPONENT_TYPE) {
			((ComponentTypeController)controller).removeInterfaceDefinition(definitionName.getName(), true);
			return Status.OK_STATUS;
		}
		return Status.CANCEL_STATUS;
	}

}
