/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.editor.operations;

import java.util.HashMap;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;
import kumbang.core.model.exception.TypeAlreadyDefinedException;
import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.editor.KumbangModellerEditor;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

public class NewAttributeValueTypeOperation extends KumbangModelOperation {

	private Name name;
	private HashMap<String, Name> values;

	public NewAttributeValueTypeOperation(ModelController controller, Name name, HashMap<String, Name> names) {
		super("New Attribute Value", controller);
		this.name = name;
		this.values = names;
	}

	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		try {
			controller.createNewAttributValueType(name, values);
		} catch (PropertyAlreadyDefinedException e) {
			throw new ExecutionException(e.getMessage(), e);
		} catch (TypeAlreadyDefinedException e) {
			throw new ExecutionException(e.getMessage(), e);
		}
		return Status.OK_STATUS;
	}


	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		controller.removeAttributeValue(name.getName(), true);;
		return Status.OK_STATUS;
	}

}
