/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.editor.operations.umlexport;

import java.io.File;
import java.util.Iterator;

import kumbang.core.model.core.KumbangModel;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ControllerFactory;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.PartDefinitionController;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Realization;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.resource.UMLResource;


public class UmlExportUtils extends UmlExportBase {

	protected static Model createModel(String name) {
		Model model = UMLFactory.eINSTANCE.createModel();
		model.setName(name);

		out("Model '" + model.getQualifiedName() + "' created.");

		return model;
	}

	protected static org.eclipse.uml2.uml.Package createPackage(
			org.eclipse.uml2.uml.Package nestingPackage, String name) {
		org.eclipse.uml2.uml.Package package_ = nestingPackage
			.createNestedPackage(name);

		out("Package '" + package_.getQualifiedName() + "' created.");

		return package_;
	}

	protected static PrimitiveType createPrimitiveType(
			org.eclipse.uml2.uml.Package package_, String name) {
		PrimitiveType primitiveType = (PrimitiveType) package_
			.createOwnedPrimitiveType(name);

		out("Primitive type '" + primitiveType.getQualifiedName()
			+ "' created.");

		return primitiveType;
	}

	protected static Enumeration createEnumeration(
			org.eclipse.uml2.uml.Package package_, String name) {
		Enumeration enumeration = (Enumeration) package_
			.createOwnedEnumeration(name);

		out("Enumeration '" + enumeration.getQualifiedName() + "' created.");

		return enumeration;
	}

	protected static EnumerationLiteral createEnumerationLiteral(
			Enumeration enumeration, String name) {
		EnumerationLiteral enumerationLiteral = enumeration
			.createOwnedLiteral(name);

		out("Enumeration literal '" + enumerationLiteral.getQualifiedName()
			+ "' created.");

		return enumerationLiteral;
	}

	protected static org.eclipse.uml2.uml.Component createComponent(
			org.eclipse.uml2.uml.Package package_, String name) {
		
		org.eclipse.uml2.uml.Component component_ = UMLFactory.eINSTANCE.createComponent();
		component_.setPackage(package_);
		component_.setName(name);
		
		return component_;
	}
	
	protected static PackageableElement createComponent(
			org.eclipse.uml2.uml.PackageableElement parent_, String name) {
		
		// There's no createNestedComponent -convinience methods so it's necessary
		// to first create a Component object and then pass it to the parent component's
		// createPackagedElement-method to make it nested
		
		// Create the component
		org.eclipse.uml2.uml.Component component_ = UMLFactory.eINSTANCE.createComponent();

		// Make a packageable element out of it
		PackageableElement pe = ((Component)parent_).createPackagedElement(name, component_.eClass());
		
		out("Component '" + pe.getQualifiedName() + "' created under " + parent_.getName());
		
		return pe;
	}
	
	protected static PackageableElement createInterface(PackageableElement parent_, String name) {
		org.eclipse.uml2.uml.Interface interface_ = ((Component)parent_).createOwnedInterface(name); 
		
		return interface_;
	}

	private static PackageableElement createUsage(PackageableElement parent_, Interface interface_) {
//		System.out.println("Creating usage for " + interface_.getName());
		//Usage usage_ = UMLFactory.eINSTANCE.createUsage();
		Component c = (Component)parent_;
		Usage usage = (Usage) c.createPackagedElement(null, UMLPackage.Literals.USAGE);
		usage.getClients().add(parent_);
		usage.getSuppliers().add(interface_);
		
		
////		
////		((Component)parent_).create
//		usage_.setName(interface_.getName());
//		usage_.getClients().add(parent_);
//		usage_.getSuppliers().add(interface_);
////		((Component)parent_).getOwnedMembers().add(usage_);
////		interface_.create
////		((Component)parent_).create
//		
////		PackageableElement pe = ((Component)parent_).createPackagedElement(interface_.getName(), usage_.eClass());
//		
//
////		  (Usage).getClients().add(Component);
////		   (Usage).getSuppliers().add(Interface);
////		   (Component).getOwnedMembers().add(Usage); 
//		
		
		//return c2.get
		return usage;
		//createDependency(interface_);
		
		
		
		
		
		
		
		
	}

	private static Realization createInterfaceRealization(PackageableElement c2, Interface interface_) {
//		System.out.println("Creating realization for " + interface_.getName());
		InterfaceRealization ir = ((Component)c2).createInterfaceRealization(interface_.getName(), interface_);
//		Compone
		return ir;
		
	}
	
	
	protected static org.eclipse.uml2.uml.Class createClass(
			org.eclipse.uml2.uml.Package package_, String name,
			boolean isAbstract) {
		org.eclipse.uml2.uml.Class class_ = package_.createOwnedClass(name,
			isAbstract);

		out("Class '" + class_.getQualifiedName() + "' created.");

		return class_;
	}

	protected static Generalization createGeneralization(
			Classifier specificClassifier, Classifier generalClassifier) {
		Generalization generalization = specificClassifier
			.createGeneralization(generalClassifier);

		out("Generalization " + specificClassifier.getQualifiedName() + " ->> "
			+ generalClassifier.getQualifiedName() + " created.");

		return generalization;
	}

	protected static Property createAttribute(
			org.eclipse.uml2.uml.Class class_, String name, Type type,
			int lowerBound, int upperBound) {
		Property attribute = class_.createOwnedAttribute(name, type,
			lowerBound, upperBound);

		StringBuffer sb = new StringBuffer();

		sb.append("Attribute '");

		sb.append(attribute.getQualifiedName());

		sb.append("' : ");

		sb.append(type.getQualifiedName());

		sb.append(" [");
		sb.append(lowerBound);
		sb.append("..");
		sb.append(LiteralUnlimitedNatural.UNLIMITED == upperBound
			? "*"
			: String.valueOf(upperBound));
		sb.append("]");

		sb.append(" created.");

		out(sb.toString());

		return attribute;
	}

	protected static Association createAssociation(Type type1,
			boolean end1IsNavigable, AggregationKind end1Aggregation,
			String end1Name, int end1LowerBound, int end1UpperBound,
			Type type2, boolean end2IsNavigable,
			AggregationKind end2Aggregation, String end2Name,
			int end2LowerBound, int end2UpperBound) {

		Association association = type1.createAssociation(end1IsNavigable,
			end1Aggregation, end1Name, end1LowerBound, end1UpperBound, type2,
			end2IsNavigable, end2Aggregation, end2Name, end2LowerBound,
			end2UpperBound);

		StringBuffer sb = new StringBuffer();

		sb.append("Association ");

		if (null == end1Name || 0 == end1Name.length()) {
			sb.append('{');
			sb.append(type1.getQualifiedName());
			sb.append('}');
		} else {
			sb.append("'");
			sb.append(type1.getQualifiedName());
			sb.append(NamedElement.SEPARATOR);
			sb.append(end1Name);
			sb.append("'");
		}

		sb.append(" [");
		sb.append(end1LowerBound);
		sb.append("..");
		sb.append(LiteralUnlimitedNatural.UNLIMITED == end1UpperBound
			? "*"
			: String.valueOf(end1UpperBound));
		sb.append("] ");

		sb.append(end2IsNavigable
			? '<'
			: '-');
		sb.append('-');
		sb.append(end1IsNavigable
			? '>'
			: '-');
		sb.append(' ');

		if (null == end2Name || 0 == end2Name.length()) {
			sb.append('{');
			sb.append(type2.getQualifiedName());
			sb.append('}');
		} else {
			sb.append("'");
			sb.append(type2.getQualifiedName());
			sb.append(NamedElement.SEPARATOR);
			sb.append(end2Name);
			sb.append("'");
		}

		sb.append(" [");
		sb.append(end2LowerBound);
		sb.append("..");
		sb.append(LiteralUnlimitedNatural.UNLIMITED == end2UpperBound
			? "*"
			: String.valueOf(end2UpperBound));
		sb.append("]");

		sb.append(" created.");

		out(sb.toString());

		return association;
	}

	public static void export(KumbangModel model, ControllerFactory cf, String fileName) {
		registerResourceFactories();
		Model umlModel = createModel(model.getName() + "Model");
		org.eclipse.uml2.uml.Package topPackage = createPackage(umlModel, model.getName() + "Package");
		
		ComponentType root = model.getComponentRoot();
		
		// TODO: Throw model exception if the root is null!
		
		org.eclipse.uml2.uml.Component c1 = createComponent(topPackage, root.getDisplayName());
		
		ComponentTypeController ctc = (ComponentTypeController) cf.createController(root);
		
		handleComponent(ctc, c1);

		if (!fileName.substring(fileName.lastIndexOf(".")).equalsIgnoreCase(".uml"))
			fileName += ".uml";
		save(umlModel, URI.createURI("file://" + fileName));
		
	}
	
	private static void handleComponent(ComponentTypeController ctc, PackageableElement c2) {
		if (ctc.hasChildren()) {
			Object[] children = ctc.getChildren();
			for (int i = 0; i < children.length; i++) {
				if (children[i] instanceof PartDefinitionController) {
					Object[] components = ((PartDefinitionController)children[i]).getChildren();
					for (int j = 0; j < components.length; j++) {
						if (components[j] instanceof ComponentTypeController) {
							PackageableElement c = createComponent(c2, ((ComponentTypeController)components[j]).getDisplayName());
							handleComponent((ComponentTypeController)components[j], c);
						}
					}
				} else if (children[i] instanceof InterfaceDefinitionController) {
					String[] interfaces = ((InterfaceDefinitionController)children[i]).getInterfaces();
				    StringBuffer sb = new StringBuffer();


					for (int j = 0; j < interfaces.length; j++) {
//						System.out.println(c2.getName() + " has interface: " + interfaces[j]);
	//					((InterfaceDefinitionController)children[i]).isGrounded();
		//				PackageableElement iface = createInterface(c2, interfaces[j]);
					    sb.append(interfaces[j]);
					}

					
					//System.out.println(((InterfaceDefinitionController)children[i]).getName());
					sb.append(" ");
					sb.append(((InterfaceDefinitionController)children[i]).getDisplayName());
					
					Interface interf = (Interface) createInterface(c2, sb.toString());
				    

		            
					if (((InterfaceDefinitionController)children[i]).isProvides()) {
						// relization
						createInterfaceRealization(c2, interf);
					} else {
						createUsage(c2, interf);
					}
					
				}
			}
		}
	}




	public static void main(String[] args) {
		registerResourceFactories();
		Model testModel = createModel("TestModel");
		
		org.eclipse.uml2.uml.Package topPackage = createPackage(testModel, "WeatherDemo_package");
		
		org.eclipse.uml2.uml.PackageableElement comp1 = createComponent(topPackage, "WeatherDemoComponent");
		
		PackageableElement c2 = createComponent(comp1, "SubComponent");
		
		PackageableElement c3= createComponent(c2, "SubsubComponent");
		PackageableElement c4= createComponent(c3, "SubsubsubComponent");
		
		 
		
		
		
//		if (1 != args.length) {
//			err("Usage: java GettingStartedWithUML2 <URI>");
//			System.exit(1);
//		}
//
		
//
//		out("Creating model...");
//
//		Model epo2Model = createModel("epo2");
//
//		out("Creating primitive types...");
//
//		PrimitiveType intPrimitiveType = createPrimitiveType(epo2Model, "int");
//		PrimitiveType stringPrimitiveType = createPrimitiveType(epo2Model,
//			"String");
//		PrimitiveType datePrimitiveType = createPrimitiveType(epo2Model, "Date");
//		PrimitiveType skuPrimitiveType = createPrimitiveType(epo2Model, "SKU");
//
//		out("Creating enumerations...");
//
//		Enumeration orderStatusEnumeration = createEnumeration(epo2Model,
//			"OrderStatus");
//
//		out("Creating enumeration literals...");
//
//		createEnumerationLiteral(orderStatusEnumeration, "Pending");
//		createEnumerationLiteral(orderStatusEnumeration, "BackOrder");
//		createEnumerationLiteral(orderStatusEnumeration, "Complete");
//
//		out("Creating classes...");
//
//		org.eclipse.uml2.uml.Class supplierClass = createClass(epo2Model,
//			"Supplier", false);
//		org.eclipse.uml2.uml.Class customerClass = createClass(epo2Model,
//			"Customer", false);
//		org.eclipse.uml2.uml.Class purchaseOrderClass = createClass(epo2Model,
//			"PurchaseOrder", false);
//		org.eclipse.uml2.uml.Class itemClass = createClass(epo2Model, "Item",
//			false);
//		org.eclipse.uml2.uml.Class addressClass = createClass(epo2Model,
//			"Address", true);
//		org.eclipse.uml2.uml.Class usAddressClass = createClass(epo2Model,
//			"USAddress", false);
//		org.eclipse.uml2.uml.Class globalAddressClass = createClass(epo2Model,
//			"GlobalAddress", false);
//		org.eclipse.uml2.uml.Class globalLocationClass = createClass(epo2Model,
//			"GlobalLocation", false);
//
//		out("Creating generalizations...");
//
//		createGeneralization(usAddressClass, addressClass);
//		createGeneralization(globalAddressClass, addressClass);
//		createGeneralization(globalAddressClass, globalLocationClass);
//
//		out("Creating attributes...");
//
//		createAttribute(supplierClass, "name", stringPrimitiveType, 0, 1);
//		createAttribute(customerClass, "customerID", intPrimitiveType, 0, 1);
//		createAttribute(purchaseOrderClass, "comment", stringPrimitiveType, 0,
//			1);
//		createAttribute(purchaseOrderClass, "orderDate", datePrimitiveType, 0,
//			1);
//		createAttribute(purchaseOrderClass, "status", orderStatusEnumeration,
//			0, 1);
//		createAttribute(purchaseOrderClass, "totalAmount", intPrimitiveType, 0,
//			1);
//		createAttribute(itemClass, "productName", stringPrimitiveType, 0, 1);
//		createAttribute(itemClass, "quantity", intPrimitiveType, 0, 1);
//		createAttribute(itemClass, "USPrice", intPrimitiveType, 0, 1);
//		createAttribute(itemClass, "comment", stringPrimitiveType, 0, 1);
//		createAttribute(itemClass, "shipDate", datePrimitiveType, 0, 1);
//		createAttribute(itemClass, "partNum", skuPrimitiveType, 0, 1);
//		createAttribute(addressClass, "name", stringPrimitiveType, 0, 1);
//		createAttribute(addressClass, "country", stringPrimitiveType, 0, 1);
//		createAttribute(usAddressClass, "street", stringPrimitiveType, 0, 1);
//		createAttribute(usAddressClass, "city", stringPrimitiveType, 0, 1);
//		createAttribute(usAddressClass, "state", stringPrimitiveType, 0, 1);
//		createAttribute(usAddressClass, "zip", intPrimitiveType, 0, 1);
//		createAttribute(globalAddressClass, "location", stringPrimitiveType, 0,
//			LiteralUnlimitedNatural.UNLIMITED);
//		createAttribute(globalLocationClass, "countryCode", intPrimitiveType,
//			0, 1);
//
//		out("Creating associations...");
//
//		createAssociation(supplierClass, true,
//			AggregationKind.COMPOSITE_LITERAL, "orders", 0,
//			LiteralUnlimitedNatural.UNLIMITED, purchaseOrderClass, false,
//			AggregationKind.NONE_LITERAL, "", 1, 1);
//
//		createAssociation(supplierClass, true, AggregationKind.NONE_LITERAL,
//			"pendingOrders", 0, LiteralUnlimitedNatural.UNLIMITED,
//			purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 0, 1);
//
//		createAssociation(supplierClass, true, AggregationKind.NONE_LITERAL,
//			"shippedOrders", 0, LiteralUnlimitedNatural.UNLIMITED,
//			purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 0, 1);
//
//		createAssociation(supplierClass, true,
//			AggregationKind.COMPOSITE_LITERAL, "customers", 0,
//			LiteralUnlimitedNatural.UNLIMITED, customerClass, false,
//			AggregationKind.NONE_LITERAL, "", 1, 1);
//
//		createAssociation(customerClass, true, AggregationKind.NONE_LITERAL,
//			"orders", 0, LiteralUnlimitedNatural.UNLIMITED, purchaseOrderClass,
//			true, AggregationKind.NONE_LITERAL, "customer", 1, 1);
//
//		createAssociation(purchaseOrderClass, true,
//			AggregationKind.NONE_LITERAL, "previousOrder", 0, 1,
//			purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 0, 1);
//
//		createAssociation(purchaseOrderClass, true,
//			AggregationKind.COMPOSITE_LITERAL, "items", 0,
//			LiteralUnlimitedNatural.UNLIMITED, itemClass, true,
//			AggregationKind.NONE_LITERAL, "order", 1, 1);
//
//		createAssociation(purchaseOrderClass, true,
//			AggregationKind.COMPOSITE_LITERAL, "billTo", 1, 1, addressClass,
//			false, AggregationKind.NONE_LITERAL, "", 1, 1);
//
//		createAssociation(purchaseOrderClass, true,
//			AggregationKind.COMPOSITE_LITERAL, "shipTo", 0, 1, addressClass,
//			false, AggregationKind.NONE_LITERAL, "", 1, 1);

		out("Saving model...");

		save(testModel, URI.createURI("file:///c:/").appendSegment("ExtendedPO2")
			.appendFileExtension(UMLResource.FILE_EXTENSION));
	}

	
}
