/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;

import kumbang.modeller.controllers.AttributeDefinitionController;
import kumbang.modeller.controllers.ModelController;

public class AttributeDefinitionPropertySource extends NamePropertySource {

	private static final String PROPERTY_ATTRIBUTE_TYPE = "kumbang.properties.attribute_type";
	private AttributeDefinitionController definition;
	private String[] attributes;
	public AttributeDefinitionPropertySource(AttributeDefinitionController def, ModelController modelController) {	
		super(modelController);
		definition = def;
	}

	
	 protected void createDescriptors() {
		 super.createDescriptors();
		 attributes = modelController.getAttributeTypes();
		 ComboBoxPropertyDescriptor cbpd = new ComboBoxPropertyDescriptor(PROPERTY_ATTRIBUTE_TYPE, "Attribute", attributes);
		 
		 propertyDescriptors.add(cbpd);
	 }
	
	public Object getEditableValue() {
		return definition;
	}

	public Object getPropertyValue(Object id) {
		Object o =  super.getPropertyValue(id);
		if (o != null)
			return o;
		
		if (PROPERTY_ATTRIBUTE_TYPE.equals(id)) {
			String name = definition.getAttributeName();
			for(int i = 0; i < attributes.length; i++)
				if (attributes[i].equals(name))
					return i;
		}
				
		
		return null;
	}

	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {
		
	}

	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);
		if (id.equals(PROPERTY_ATTRIBUTE_TYPE) && value instanceof Integer) {
			definition.setTypes(attributes[((Integer)value).intValue()]);
		}
			
		
	}

	protected String getName() {
		return definition.getName();
	}

	@Override
	protected void setName(String name) {
		definition.setName(name);
		
	}

	@Override
	protected String getDisplayName() {
		return definition.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		definition.setDisplayName(displayName);	
	}

	
}
