/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import kumbang.core.model.type.Name;
import kumbang.modeller.controllers.AttributeValueTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.event.ModelChangeEvent;
import kumbang.modeller.ui.event.ModellerEvent;
import kumbang.modeller.ui.properties.descriptors.AttributeDialogPropertyDescriptor;

public class AttributeValueTypePropertySource extends NamePropertySource {


	
	
	private static final String PROPERTY_ATTRIBUTE_VALUE = "kumbang.property.attribute_value";
	private AttributeValueTypeController type;

	public AttributeValueTypePropertySource(AttributeValueTypeController type, ModelController modelController) {	
		super(modelController);
		this.type = type;
	}
	
	 protected void createDescriptors() {
		 super.createDescriptors();	
		 for (int i = 0; i < type.getValues().length; i++) {
			 propertyDescriptors.add(new TextPropertyDescriptor(PROPERTY_ATTRIBUTE_VALUE + i, "Value"));
		 }
	 }

	public Object getEditableValue() {
		return type;
	}

	public Object getPropertyValue(Object id) {
		Object value = super.getPropertyValue(id);
		if (value == null && id instanceof String && ((String)id).startsWith(PROPERTY_ATTRIBUTE_VALUE)) {
			String intString = ((String)id).substring(PROPERTY_ATTRIBUTE_VALUE.length());
			return type.getValues()[Integer.parseInt(intString)].getName();
		}
		return value;
	}

	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	protected String getName() {
		return type.getName();
	}
	
	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);
		
		if (value instanceof String && id instanceof String && ((String)id).startsWith(PROPERTY_ATTRIBUTE_VALUE)) {
			String intString = ((String)id).substring(PROPERTY_ATTRIBUTE_VALUE.length());
			Name currentValue = type.getValues()[Integer.parseInt(intString)];
			currentValue.setName((String) value);
			modelController.modified(new ModelChangeEvent());
		}

	}
	
	@Override
	protected void setName(String string) {
		type.setName(string);
		
	}
	
	@Override
	protected String getDisplayName() {
		return type.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		type.setDisplayName(displayName);
		
	}
}
