/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.FeatureType;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.ModelController;

public class FeatureTypePropertySource extends NamePropertySource {


	private static final String PROPERTY_SUPERTYPE = "kumbang.property.supertype";
	private FeatureTypeController type;
	private List<ComposableType> supertypes = new ArrayList<ComposableType>();

	public FeatureTypePropertySource(FeatureTypeController type, ModelController modelController) {	
		super(modelController);
		this.type = type;
	
	}
	
	 protected void createDescriptors() {
		 super.createDescriptors(); 
		 
		 int index = 0;
		 
		 for (Iterator<ComposableType> it = type.getSupertypes(); it.hasNext(); ) {
			 supertypes.add(it.next());
			 TextPropertyDescriptor tpd = new TextPropertyDescriptor(PROPERTY_SUPERTYPE + index, "Supertype");
			
			 propertyDescriptors.add(tpd);
			 index++;
		 }
	 }
	 

	public Object getEditableValue() {
		return type;
	}
	

	public Object getPropertyValue(Object id) {
		Object value = super.getPropertyValue(id);
		if (value == null && id instanceof String && ((String)id).startsWith(PROPERTY_SUPERTYPE)) {
			String intString = ((String)id).substring(PROPERTY_SUPERTYPE.length());
			return supertypes.get(Integer.parseInt(intString)).getName();
		}
		return value;
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		

	}
	
	protected String getName() {
		return type.getName();
	}

	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);

	}
	
	@Override
	protected void setName(String string) {
		type.setName(string);
		
	}
	
	@Override
	protected String getDisplayName() {
		return type.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		type.setDisplayName(displayName);
		
	}
	
}
