/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import kumbang.core.model.type.InterfaceDefinition;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.properties.descriptors.BooleanPropertyDescriptor;

public class InterfaceDefinitionPropertySource extends NamePropertySource {

	private static final String PROPERTY_POTENTIAL_TYPE = "kumbang.property.potential_interface";
	private InterfaceDefinitionController definition;

	public InterfaceDefinitionPropertySource(InterfaceDefinitionController def, ModelController modelController) {	
		super(modelController);
		this.definition = def;
	}

	 protected void createDescriptors() {
		 super.createDescriptors();
		 

		for (int i = 0; i < definition.getInterfaces().length; i++) {
				 propertyDescriptors.add(new TextPropertyDescriptor(PROPERTY_POTENTIAL_TYPE + i, "Potential Type"));
			}
		 
		BooleanPropertyDescriptor grounded = new BooleanPropertyDescriptor(PROPERTY_GROUNDED, "Grounded");
		propertyDescriptors.add(grounded);
		
		BooleanPropertyDescriptor optional = new BooleanPropertyDescriptor(PROPERTY_OPTIONAL, "Optional");
		propertyDescriptors.add(optional);
	 }
	
	public Object getEditableValue() {
		return definition;
	}

	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);
		if (id == PROPERTY_GROUNDED && value instanceof Boolean)
			definition.setGrounded(((Boolean)value).booleanValue());
		if (id == PROPERTY_OPTIONAL && value instanceof Boolean) 	
			definition.setOptional(((Boolean)value).booleanValue());
		
	}
	public Object getPropertyValue(Object id) {
		if (id instanceof String && ((String)id).startsWith(PROPERTY_POTENTIAL_TYPE)) {
			String intString = ((String)id).substring(PROPERTY_POTENTIAL_TYPE.length());
			return definition.getInterfaces()[Integer.parseInt(intString)];
		}
		
		if (id == PROPERTY_GROUNDED)
			return Boolean.valueOf(definition.isGrounded());
		if (id == PROPERTY_OPTIONAL)
			return Boolean.valueOf(definition.isOptional());
		return super.getPropertyValue(id);
	}

	protected String getName() {
		return definition.getName();
	}

	@Override
	protected void setName(String name) {
		definition.setName(name);
		
	}
	
	@Override
	protected String getDisplayName() {
		return definition.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		definition.setDisplayName(displayName);
		
	}


	
	
}
