/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import java.util.Iterator;

import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceType;
import kumbang.modeller.controllers.InterfaceTypeController;
import kumbang.modeller.controllers.ModelController;

public class InterfaceTypePropertySource extends NamePropertySource {

	private static final String PROPERTY_FUNCTION = "kumbang.property.function";
	private InterfaceTypeController type;
	
	public InterfaceTypePropertySource(InterfaceTypeController type, ModelController modelController) {	
		super(modelController);
		this.type = type;
	}
	
	 protected void createDescriptors() {
		 super.createDescriptors(); 
		 
		 for (int i = 0; i < type.getFunctions().length; i++) {
			 propertyDescriptors.add(new TextPropertyDescriptor(PROPERTY_FUNCTION + i, "Function"));
		 }
	 }
	
	public Object getEditableValue() {
		return type;
	}

	public Object getPropertyValue(Object id) {
		Object value = super.getPropertyValue(id);
		if (value == null && id instanceof String && ((String)id).startsWith(PROPERTY_FUNCTION)) {
			String intString = ((String)id).substring(PROPERTY_FUNCTION.length());
			return type.getFunctions()[Integer.parseInt(intString)];
		}
		return value;
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}
	
	protected String getName() {
		return type.getName();
	}


	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);

	}
	
	@Override
	protected void setName(String string) {
		type.setName(string);
		
	}
	@Override
	protected String getDisplayName() {
		return type.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		type.setDisplayName(displayName);
		
	}

}
