/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties;

import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.controllers.PartDefinitionController;
import kumbang.modeller.ui.properties.descriptors.NumberPropertyDescriptor;

import org.eclipse.ui.views.properties.TextPropertyDescriptor;

public  class PartDefinitionPropertySource extends NamePropertySource {

	private static final String PROPERTY_POTENTIAL_TYPE = "kumbang.property.potential_type";
	private PartDefinitionController definition;

	public PartDefinitionPropertySource(PartDefinitionController def, ModelController modelController) {	
		super(modelController);
		this.definition = def;
		
	}

	public Object getEditableValue() {
		return definition;
	}


	protected void createDescriptors() {
		super.createDescriptors();	
		
		NumberPropertyDescriptor lowerBound = new NumberPropertyDescriptor(PROPERTY_LOWER_BOUND, "Lower bound");
		NumberPropertyDescriptor upperBound = new NumberPropertyDescriptor(PROPERTY_UPPER_BOUND, "Upper bound");
//		lowerBound.setCategory("Cardinality");
//		upperBound.setCategory("Cardinality");
		
		for (int i = 0; i < definition.getPotentialTypes().length; i++) {
			 propertyDescriptors.add(new TextPropertyDescriptor(PROPERTY_POTENTIAL_TYPE + i, "Potential Type"));
		 }
		
		propertyDescriptors.add(lowerBound);
		propertyDescriptors.add(upperBound);
	}
	
	
	public Object getPropertyValue(Object id) {
		
		Object value = super.getPropertyValue(id);
		if (value != null)
			return value;
		
		if (id instanceof String && ((String)id).startsWith(PROPERTY_POTENTIAL_TYPE)) {
			String intString = ((String)id).substring(PROPERTY_POTENTIAL_TYPE.length());
			return definition.getPotentialTypes()[Integer.parseInt(intString)];
		}
		
		if (id == PROPERTY_LOWER_BOUND)
			return definition.getLowerBound() + "";
		
		
		if (id == PROPERTY_UPPER_BOUND)
			return definition.getUpperBound() + "";
		
		return super.getPropertyValue(id);
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub
		
	}

	public void setPropertyValue(Object id, Object value) {
		super.setPropertyValue(id, value);
		if (id == PROPERTY_LOWER_BOUND && value instanceof Integer)
			definition.setLowerBound(((Integer)value).intValue());
		if (id == PROPERTY_UPPER_BOUND && value instanceof Integer)
			definition.setUpperBound(((Integer)value).intValue());
		
	}
	

	protected String getName() {
		return definition.getName();
	}

	@Override
	protected void setName(String name) {
		definition.setName(name);
		
	}
	
	@Override
	protected String getDisplayName() {
		return definition.getDisplayName();
	}

	@Override
	protected void setDisplayName(String displayName) {
		definition.setDisplayName(displayName);
		
	}


}
