/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.properties.descriptors;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.views.properties.PropertyDescriptor;

public class NumberPropertyDescriptor extends PropertyDescriptor {

	private class NumberCellEditor extends CellEditor {

		private Spinner spinner;

		public NumberCellEditor(Composite parent, int style) {
			super(parent, style);
		}

		@Override
		protected Control createControl(Composite parent) {
			spinner = new Spinner(parent, SWT.NULL );
			spinner.setMinimum(0);
			return spinner;
		}

		@Override
		protected Object doGetValue() {
			return spinner.getSelection();
		}

		@Override
		protected void doSetFocus() {
			spinner.setFocus();
			
		}

		@Override
		protected void doSetValue(Object value) {
			if (value instanceof Integer)
				spinner.setSelection(((Integer)value).intValue());
			else if (value instanceof String) {
				try {
				spinner.setSelection(Integer.parseInt((String)value));
				} catch(NumberFormatException nfe) {}
			}
			
		}
		
	}
	
	
	public NumberPropertyDescriptor(Object id, String displayName) {
		super(id, displayName);
		
	}

	 public CellEditor createPropertyEditor(Composite parent) {
		return new NumberCellEditor(parent, SWT.NULL);
		 
	 }
}
