/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import kumbang.core.model.core.ForfamelModel;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComponentTypeController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ConnectionGroupController;
import kumbang.modeller.controllers.ConstraintGroupController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.InterfaceDefinitionController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.AttributeDefinitionDialog;
import kumbang.modeller.ui.dialogs.InterfaceDefinitionDialog;
import kumbang.modeller.ui.dialogs.PartDefinitionDialog;
import kumbang.modeller.ui.dnd.AttributeDefinitionTransfer;
import kumbang.modeller.ui.dnd.AttributeTypeTransfer;
import kumbang.modeller.ui.dnd.ComponentTypeTransfer;
import kumbang.modeller.ui.dnd.ComponentViewDragSourceListener;
import kumbang.modeller.ui.dnd.ComponentViewerDropAdapter;
import kumbang.modeller.ui.dnd.FeatureTypeTransfer;
import kumbang.modeller.ui.dnd.FeatureViewDragSourceListener;
import kumbang.modeller.ui.dnd.FeatureViewerDropAdapter;
import kumbang.modeller.ui.dnd.InterfaceDefinitionTransfer;
import kumbang.modeller.ui.dnd.InterfaceTypeTransfer;
import kumbang.modeller.ui.dnd.PartDefinitionTransfer;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;

public class ComponentTreeViewer extends KumbangTreeViewer {


	public ComponentTreeViewer(Composite parent, int style,
			ModelController modelController, IUndoContext undoContext, ViewerActionFactory actionFactory, IBaseLabelProvider labelProvider) {
		super(parent, style, modelController, undoContext, actionFactory );
		int operations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_DEFAULT;
		addDragSupport(operations, new Transfer[]{PartDefinitionTransfer.getInstance(), AttributeDefinitionTransfer.getInstance(), InterfaceDefinitionTransfer.getInstance()}, new ComponentViewDragSourceListener(this ));
		addDropSupport(operations, new Transfer[]{ComponentTypeTransfer.getInstance(), AttributeTypeTransfer.getInstance(), InterfaceTypeTransfer.getInstance(), PartDefinitionTransfer.getInstance(), InterfaceDefinitionTransfer.getInstance(), AttributeDefinitionTransfer.getInstance()}, new ComponentViewerDropAdapter(this, modelController, undoContext));	
		
		//setAutoExpandLevel(3);
		setContentProvider(new ComponentContentProvider(modelController.getControllerFactory()));
		setLabelProvider(labelProvider);
		setInput(modelController.getModel());
	}

	protected void fillSpecificContextMenu(IMenuManager mgr) {
		BasicController selection = getSelectedController();
		if (selection == null) {
			
			if (!modelController.isComponentRootSet())
				mgr.add(actionFactory.getCreateComponentRootAction());
		
			return;
		}
		
		
		
		if (selection.getType() ==  BasicController.COMPONENT_TYPE) {
			mgr.add(actionFactory.getAddPartDefinitionAction((ComposableTypeController) selection, "Add Part Definition..."));
			mgr.add(actionFactory.getAddAttributeDefinitionAction((ComposableTypeController) selection));
			mgr.add(actionFactory.getAddInterfaceDefinitionAction((ComposableTypeController) selection));
			Action connect = actionFactory.getAddConnectionAction((ComponentTypeController) selection);
			mgr.add(connect);
			mgr.add(actionFactory.getAddConstraintAction((ComponentTypeController)selection));
			
		}
		
//		if (selection.getType() == BasicController.PART_DEFINITION) {
//			mgr.add(actionFactory.getAddPossibleTypesAction());
//		}
		
		if (selection.getType() ==  BasicController.CONSTRAINT_GROUP) {
			mgr.add(actionFactory.getAddConstraintAction(((ConstraintGroupController)selection).getParentController()));
		
		}
		if (selection.getType() ==  BasicController.CONNECTION_GROUP) {
			mgr.add(actionFactory.getAddConnectionAction((ComponentTypeController) ((ConnectionGroupController)selection).getParentController()));
		
		}
		
	
		
	}

	@Override
	protected void updateModelStatus() {
		if (modelController.getModelType().equals(ForfamelModel.MODEL_KIND)) 
			getTree().setEnabled(false);
		else 
			getTree().setEnabled(true);
		
	}

}
