/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import kumbang.core.model.core.KoalishModel;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ComposableTypeController;
import kumbang.modeller.controllers.ConstraintGroupController;
import kumbang.modeller.controllers.FeatureTypeController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dnd.AttributeDefinitionTransfer;
import kumbang.modeller.ui.dnd.AttributeTypeTransfer;
import kumbang.modeller.ui.dnd.FeatureTypeTransfer;
import kumbang.modeller.ui.dnd.FeatureViewDragSourceListener;
import kumbang.modeller.ui.dnd.FeatureViewerDropAdapter;
import kumbang.modeller.ui.dnd.PartDefinitionTransfer;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;

public class FeatureTreeViewer extends KumbangTreeViewer {

	
	public FeatureTreeViewer(Composite parent, int style, ModelController modelController, IUndoContext undoContext, ViewerActionFactory actionFactory, KumbangLabelProvider provider) {
		super(parent, style, modelController, undoContext, actionFactory);
		int operations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_DEFAULT;
		addDragSupport(operations, new Transfer[]{PartDefinitionTransfer.getInstance(), AttributeDefinitionTransfer.getInstance()}, new FeatureViewDragSourceListener(this ));
		addDropSupport(operations, new Transfer[]{FeatureTypeTransfer.getInstance(), AttributeTypeTransfer.getInstance(), PartDefinitionTransfer.getInstance(), AttributeDefinitionTransfer.getInstance()}, new FeatureViewerDropAdapter(this, modelController, undoContext));	
		
		//setAutoExpandLevel(3);
		setContentProvider(new FeatureContentProvider(modelController.getControllerFactory()));
		setLabelProvider(provider);
		setInput(modelController.getModel());
		
		
	
	}

	protected void fillSpecificContextMenu(IMenuManager manager) {
		BasicController selection = getSelectedController();
		if (selection == null) {
			
			if (!modelController.isFeatureRootSet())
				manager.add(actionFactory.getCreateFeatureRootAction());
		
			return;
		}
		if (selection.getType() ==  BasicController.FEATURE_TYPE) {
		
			manager.add(actionFactory.getAddPartDefinitionAction((ComposableTypeController) getSelectedController(), "Add Subfeature..."));
			manager.add(actionFactory.getAddAttributeDefinitionAction((ComposableTypeController) getSelectedController()));
			manager.add(actionFactory.getAddConstraintAction((FeatureTypeController)selection));
			manager.add(actionFactory.getAddImplementationConstraintAction((FeatureTypeController)selection));
			
			
		}	
		if (selection.getType() ==  BasicController.CONSTRAINT_GROUP) {
			manager.add(actionFactory.getAddConstraintAction(((ConstraintGroupController)selection).getParentController()));
			manager.add(actionFactory.getAddImplementationConstraintAction((FeatureTypeController) ((ConstraintGroupController)selection).getParentController()));
		}
		
	}
	
	
	@Override
	protected void updateModelStatus() {
		if (modelController.getModelType().equals(KoalishModel.MODEL_KIND)) 
			getTree().setEnabled(false);
		else 
			getTree().setEnabled(true);
		
	}
}
	

