/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;


import kumbang.modeller.KumbangModellerPlugin;
import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.util.Constants;

import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IFontProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.internal.OverlayIcon;


public class KumbangLabelProvider extends LabelProvider implements  IColorProvider, IFontProvider, Constants {


	private static KumbangLabelProvider provider;
	private ImageRegistry images; 
	private ColorRegistry colors; 
	
	
	
	private KumbangLabelProvider() {
		images = KumbangModellerPlugin.getDefault().getImageRegistry();
		colors = KumbangModellerPlugin.getDefault().getColorRegistry();
	}
	
	public static KumbangLabelProvider getInstance() {
		if (provider == null)
			provider = new KumbangLabelProvider();
		
		return provider;
	}
	
	
	public Image getImage(Object element) {
		String imageString = null;
		if (element instanceof BasicController)
			imageString = ((BasicController)element).getImageString();			
		return getRealImage(imageString);
		
	}
	

	public ImageDescriptor getImageDescriptor(String imageString) {
		ImageDescriptor id = images.getDescriptor(imageString);
		
		if (id == null) {
			id = KumbangModellerPlugin.getImageDescriptor(imageString);
			images.put(imageString, id);
			
		}
		
		return id;
	}
	
	public Image getRealImage(String imageString) {
		if (imageString == null)
			return null;
		
		Image image = images.get(imageString);
		if (image == null) {
			images.put(imageString, KumbangModellerPlugin.getImageDescriptor(imageString));
		}
		
		return images.get(imageString);
	}

	public String getText(Object element) {
		if (element instanceof BasicController) {
			return ((BasicController)element).getDisplayText();
		}
		
		if (element != null)
			return element.toString();
		
		return null;
	}

	public Color getBackground(Object element) {
//		if (element instanceof IController) {
//			return ((IController)element).getForeground();
//		}
		return null;
	}

	public Color getForeground(Object element) {
		if (element instanceof BasicController) {
			
			return colors.get(((BasicController)element).getBackgroundString());
//			if (((BasicController)element).getLevel() %2 == 0 || ((BasicController)element).getLevel() < 0 )
//				return colors.get("LIGHT_GREEN");
//			else	
//				return colors.get("LIGHT_BLUE");
		}
		return null;
	}

	public Font getFont(Object element) {
//		if (element instanceof IController) {
//			return ((IController)element).getFont();
//		}
		return null;
	}

	public Color getColor(int type) {
		switch(type){
		case FEATURE_TYPE:
			return colors.get(FEATURE_TYPE_COLOR_STRING);
		case COMPONENT_TYPE:
			return colors.get(COMPONENT_TYPE_COLOR_STRING);
		case INTERFACE_TYPE:
			return colors.get(INTERFACE_TYPE_COLOR_STRING);
		case ATTRIBUTE_VALUE_TYPE:
			return colors.get(ATTRIBUTE_VALUE_COLOR_STRING);
		}
		return null;
	}

	public Image getImage(int type) {
		String imageString = null;
		
		switch(type){
		case FEATURE_TYPE:
			imageString = FEATURE_TYPE_IMAGE_STRING;
			break;
		case COMPONENT_TYPE:
			imageString = COMPONENT_TYPE_IMAGE_STRING;
			break;
		case INTERFACE_TYPE:
			imageString = INTERFACE_TYPE_IMAGE_STRING;
			break;
		case ATTRIBUTE_VALUE_TYPE:
			imageString = ATTRIBUTE_VALUE_TYPE_IMAGE_STRING;
			break;
		}
		return getRealImage(imageString);
	}
	
}
