/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import java.util.List;

import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ModelController;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;

public abstract class KumbangTreeViewer extends TreeViewer {

	protected IUndoContext undoContext;
	protected ModelController modelController;
	protected ViewerActionFactory actionFactory;
	protected Action deleteAction;

	public KumbangTreeViewer(Composite parent, int style, ModelController modelController, IUndoContext undoContext, ViewerActionFactory af) {
		super(parent, style);
		this.modelController = modelController;
		this.undoContext = undoContext;
		this.actionFactory = af;
		//expandToLevel(1);
		setAutoExpandLevel(3);
		
		createContextMenu();
		getTree().addSelectionListener(new SelectionAdapter() {

			public void widgetDefaultSelected(SelectionEvent e) {
				actionFactory.getPropertiesAction(getSelectedController()).run();
				
			}
			
			public void widgetSelected(SelectionEvent e) {
				actionFactory.getDeleteAction(getSelectedController());
				
			}
		});
		updateModelStatus();
		
	}

	
	protected abstract void updateModelStatus();


	public void createContextMenu() {
		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				
				fillSpecificContextMenu(manager);
				manager.add(new Separator());
				fillCommonContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(getControl());
		getControl().setMenu(menu);
	
	
	}

	protected abstract void fillSpecificContextMenu(IMenuManager mgr);

	protected void fillCommonContextMenu(IMenuManager mgr) {
	
		MenuManager subMenu = new MenuManager("Create New Type...");
		mgr.add(subMenu);		
		
		subMenu.add(actionFactory.getNewFeatureTypeAction());
		subMenu.add(actionFactory.getNewComponentTypeAction());
		
		subMenu.add(actionFactory.getNewAttributeTypeAction());
		subMenu.add(actionFactory.getNewInterfaceAction());
		mgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		mgr.add(new Separator());	
		
		if (!modelController.isComponentRootSet() && modelController.getModel().getComponentCount() > 0)
			mgr.add(actionFactory.getSelectComponentRootAction());
		
		if (!modelController.isFeatureRootSet() && modelController.hasFeatureModelData())
			mgr.add(actionFactory.getSelectFeatureRootAction());
		
		mgr.add(new Separator());
		BasicController selection = getSelectedController();
		if (selection != null){
			mgr.add(actionFactory.getPropertiesAction(selection));
			mgr.add(actionFactory.getDeleteAction(selection));
		}
		mgr.add(new Separator());
//		mgr.add(new Action("{DEBUG} Print current model") {
//			
//			public void run() {
//				System.out.print(KumbangModelPrinter.getInstance().getAsString(modelController.getModel(), modelController.getModelType()));
//			}
//		});
		actionFactory.updateActionStatus();
	
	}

	
	public BasicController getSelectedController() {
		List selectionList = getSelectionFromWidget();
		if (selectionList.size() == 0 || !(selectionList.get(0) instanceof BasicController))
			return null;
		return (BasicController)selectionList.get(0);
	}

	public void refresh() {
		//Object[] expanded = this.getExpandedElements();
		super.refresh();
		//this.setExpandedElements(expanded);
		updateModelStatus();
	}


	public void setModelController(ModelController modelController) {
		this.modelController = modelController;
		setInput(modelController.getModel());
		
	}
	
}
