/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.util.Constants;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.FormToolkit;


public class TypeViewer implements Constants /*, IExpansionListener */, ISelectionProvider, ISelectionChangedListener {

	private Composite composite;
	private TypeViewerGroup featureTypeViewer;
	private ViewerActionFactory actionFactory;
	private TypeViewerGroup componentTypeViewer;
	private TypeViewerGroup attributeTypeViewer;
	private TypeViewerGroup interfaceTypeViewer;
	private ModelController modelController;
	private ListenerList listeners = new ListenerList();
	private ISelection selection;
	
	public TypeViewer(FormToolkit toolkit, Composite parent, IUndoContext undoContext, ModelController modelController, ViewerActionFactory actionFactory, KumbangLabelProvider labelProvider) {
		composite = toolkit.createComposite(parent, SWT.SCROLL_LINE);
		composite.setLayout(new GridLayout(1, true));		
		this.actionFactory = actionFactory;
		this.modelController = modelController;
		//createContextMenu();
		
		featureTypeViewer = new TypeViewerGroup(composite, toolkit, modelController, undoContext,  FEATURE_TYPE, actionFactory, labelProvider, this);		
		componentTypeViewer = new TypeViewerGroup(composite, toolkit, modelController, undoContext,  COMPONENT_TYPE, actionFactory, labelProvider, this);
		attributeTypeViewer = new TypeViewerGroup(composite, toolkit, modelController, undoContext,  ATTRIBUTE_VALUE_TYPE, actionFactory, labelProvider, this);
		interfaceTypeViewer = new TypeViewerGroup(composite, toolkit, modelController, undoContext,  INTERFACE_TYPE, actionFactory, labelProvider, this);


	
		
		updateModelType();
	}

	
	private void updateModelType() {
		if (modelController.getModelType().equals(ForfamelModel.MODEL_KIND)) {
			componentTypeViewer.setEnabled(false);
			interfaceTypeViewer.setEnabled(false);
			featureTypeViewer.setEnabled(true);
		} else if (modelController.getModelType().equals(KoalishModel.MODEL_KIND)) {
			componentTypeViewer.setEnabled(true);
			interfaceTypeViewer.setEnabled(true);
			featureTypeViewer.setEnabled(false);
		} else {
			componentTypeViewer.setEnabled(true);
			interfaceTypeViewer.setEnabled(true);
			featureTypeViewer.setEnabled(true);
		}
			
	}


	public void refresh() {
		featureTypeViewer.refresh();
		componentTypeViewer.refresh();
		attributeTypeViewer.refresh();
		interfaceTypeViewer.refresh();
		updateModelType();
		
	}
	
	public Control getControl() {
		return composite;
	}


	public void setModelController(ModelController modelController) {
		if(this.modelController == modelController)
			return; //Nothing changed
		
		this.modelController = modelController;
		featureTypeViewer.setModelController(modelController);
		componentTypeViewer.setModelController(modelController);
		attributeTypeViewer.setModelController(modelController);
		interfaceTypeViewer.setModelController(modelController);
		
	}



	public ISelectionProvider getSelectionProvider() {
		return this;
	}


	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		listeners.add(listener);
		
	}


	public ISelection getSelection() {
		return selection;
	}


	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		listeners.remove(listener);
		
	}


	public void setSelection(ISelection selection) {
		this.selection = selection;
	
	
		
	}


	public void selectionChanged(SelectionChangedEvent event) {
		setSelection(event.getSelection());
		for (Object l : listeners.getListeners()) {
			((ISelectionChangedListener)l).selectionChanged(event);
		}
		
	}
}
