/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.viewers;

import kumbang.modeller.controllers.BasicController;
import kumbang.modeller.controllers.ModelController;
import kumbang.modeller.ui.dialogs.LayoutFactory;
import kumbang.modeller.ui.dnd.AttributeTypeTransfer;
import kumbang.modeller.ui.dnd.ComponentTypeTransfer;
import kumbang.modeller.ui.dnd.FeatureTypeTransfer;
import kumbang.modeller.ui.dnd.InterfaceTypeTransfer;
import kumbang.modeller.ui.dnd.TypeViewDragSourceListener;
import kumbang.modeller.util.Constants;
import kumbang.modeller.util.KumbangModelPrinter;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.forms.widgets.FormToolkit;

public class TypeViewerGroup implements Constants {

	private ModelController modelController;
	private IUndoContext undoContext;
	private ViewerActionFactory actionFactory;
	private Group group;
	private TableViewer viewer;
	private TypeViewer tv;
	
	public TypeViewerGroup(Composite parent, FormToolkit toolkit, ModelController modelController, IUndoContext undoContext, int type, ViewerActionFactory af, KumbangLabelProvider labelProvider, TypeViewer tv) {
		
		this.group = new Group(parent, SWT.NULL);	
		this.tv = tv;
	
		group.setLayout(new FillLayout());

		toolkit.adapt(group);
		this.viewer = new TableViewer(group, SWT.SINGLE | SWT.V_SCROLL);
		//this.viewer.getTable().getVerticalBar().setVisible(true);
		this.viewer.setContentProvider(new TypeContentProvider(type));
		this.viewer.setLabelProvider(labelProvider);
		this.viewer.setInput(modelController);
		viewer.addSelectionChangedListener(tv);
		viewer.getTable().addFocusListener(new FocusAdapter() {

			// This enables correct tracking of the active selection
			public void focusGained(FocusEvent e) {
				TypeViewerGroup.this.tv.selectionChanged(new SelectionChangedEvent(viewer, viewer.getSelection()));
				
			}

		});
		
		
		this.viewer.getTable().addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				actionFactory.getPropertiesAction(getSelectedController()).run();
				
			}

			public void widgetSelected(SelectionEvent e) {
				actionFactory.getDeleteAction(getSelectedController());
				
			}
		});
		
		
		//toolkit.adapt(viewer.getTable());
	//this.section.setClient(this.viewer.getTable());
		
		
//		Label image = new Label(section, SWT.LEFT); 
////		image.setFont(this.section.getFont());
//		image.setImage(labelProvider.getImage(type));
//		image.setBackground(section.getBackground());
////		
//		this.section.setTextClient(image);
	
		
		this.modelController =  modelController;
		this.undoContext = undoContext;
		this.actionFactory = af;
			
		Transfer transfer = null;
		switch (type) {
		case FEATURE_TYPE:
			this.group.setText("Feature Types");
	
			transfer = FeatureTypeTransfer.getInstance();
			break;
		case COMPONENT_TYPE:
	
			this.group.setText("Component Types");
			transfer = ComponentTypeTransfer.getInstance();
			break;
		case ATTRIBUTE_VALUE_TYPE:
	
			this.group.setText("Attribute Value Types");
			transfer = AttributeTypeTransfer.getInstance();
			break;
		case INTERFACE_TYPE:
	
			this.group.setText("Interface Types");
			transfer = InterfaceTypeTransfer.getInstance();
			break;

		}
		
		if (transfer != null)
			viewer.addDragSupport(DND.DROP_COPY, new Transfer[] {transfer}, new TypeViewDragSourceListener(this.viewer));
		
//		this.viewer.getTable().setLayoutData(new GridData(GridData.GRAB_VERTICAL | GridData.FILL_BOTH));
		this.group.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		
		createContextMenu();
	}


	public void createContextMenu() {
		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				
				fillContextMenu(manager);
		
				
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		//section.setMenu(menu);
	
	
	}

	



	protected void fillContextMenu(IMenuManager mgr) {
		BasicController selection = getSelectedController();
		

		
		MenuManager subMenu = new MenuManager("Create New Type...");

		mgr.add(subMenu);
		
		
		subMenu.add(actionFactory.getNewFeatureTypeAction());
		subMenu.add(actionFactory.getNewComponentTypeAction());				
		subMenu.add(actionFactory.getNewInterfaceAction());
		subMenu.add(actionFactory.getNewAttributeTypeAction());
		
		
		mgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		mgr.add(new Separator());
		if (!modelController.isComponentRootSet() && modelController.getModel().getComponentCount() > 0)
			mgr.add(actionFactory.getSelectComponentRootAction());
		
		if (!modelController.isFeatureRootSet() && modelController.hasFeatureModelData())
			mgr.add(actionFactory.getSelectFeatureRootAction());

		
		if (selection != null){
			mgr.add(actionFactory.getPropertiesAction(selection));
			mgr.add(actionFactory.getDeleteAction(selection));
			
			if (selection.getType() ==  BasicController.COMPONENT_TYPE && !modelController.isComponentRootSet()) {		
				mgr.add(actionFactory.getSetAsComponentRootAction(selection.getName()));
			}
			if (selection.getType() ==  BasicController.FEATURE_TYPE && !modelController.isFeatureRootSet()){
				mgr.add(actionFactory.getSetAsFeatureRootAction(selection.getName()));
			}	
			
		}
		mgr.add(new Separator());
//		mgr.add(new Action("{DEBUG} Print current model") {
//			
//			public void run() {
//				System.out.print(KumbangModelPrinter.getInstance().getAsString(modelController.getModel(), modelController.getModelType()));
//			}
//		});

		actionFactory.updateActionStatus();
	}
	
	protected BasicController getSelectedController() {
		ISelection selection = viewer.getSelection();
		if (selection != null && selection instanceof StructuredSelection 
				&& ((StructuredSelection)selection).getFirstElement() instanceof BasicController) {
			return (BasicController) ((StructuredSelection)selection).getFirstElement();
		}
		return null;
	}


	public void refresh() {
		viewer.refresh();
	}


	public void setModelController(ModelController modelController) {
		this.modelController = modelController;
		viewer.setInput(modelController);
		refresh();
	}


	public void setEnabled(boolean b) {
		viewer.getControl().setEnabled(b);
		
	}
		
	public void setVisible(boolean b) {
		viewer.getControl().setVisible(b);
	
	}




}
