/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.views;

import kumbang.modeller.ui.editor.KumbangModellerEditor;
import kumbang.modeller.ui.event.ModelChangeListener;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.SubActionBars;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.MessagePage;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PageBookView;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetSorter;
import org.eclipse.ui.views.properties.IPropertySheetEntry;


public abstract class KumbangView extends PageBookView implements ModelChangeListener {


	private final static String defaultText = "This view offers no information for currently active editor";
	private KumbangModellerEditor editor;
	
	
	protected IPage createDefaultPage(PageBook book) {
	      MessagePage page = new MessagePage();
	      initPage(page);
	      page.createControl(book);
	      page.setMessage(defaultText);
	      return page;
		
	}
	

	
	protected void doDestroyPage(IWorkbenchPart part, PageRec pageRecord) {
		pageRecord.page.dispose();
		pageRecord.dispose();
		
	}
	
	protected void showPageRec(PageRec rec) {
		createGlobalActionBars(rec);
		super.showPageRec(rec);

	}
	
	protected void createGlobalActionBars(PageRec rec) {
		
		
		if (rec.page instanceof KumbangViewPage) {
			
			editor = ((KumbangViewPage)rec.page).getEditor();

			SubActionBars bars = rec.subActionBars;
			bars.clearGlobalActionHandlers();
			bars.deactivate();
			
			new UndoRedoActionGroup(this.getSite(), editor.getUndoContext(), false).fillActionBars(bars);

			//bars.setGlobalActionHandler(ActionFactory.DELETE.getId(),
			//	editor.getActionFactory().getDeleteAction());
			bars.updateActionBars();
			bars.activate();
		
			getViewSite().getActionBars().updateActionBars();
			
			//rec.page.setActionBars(bars);
		
		}
		
	}
	
	protected IWorkbenchPart getBootstrapPart() {
		 IWorkbenchPage page = getSite().getPage();
	      if (page != null)
	           return page.getActiveEditor();
	      return null;
	}


	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySheetPage.class) {
	      return new PropertySheetPage() {
  	  
	    	  public void createControl(Composite parent) {
	    		  setSorter(new PropertySheetSorter() {
	    			  
	    			  public int compare(IPropertySheetEntry entryA, IPropertySheetEntry entryB) {
	    					// No sorting
	    					return 0;
	    				}

	    				public int compareCategories(String categoryA, String categoryB) {
	    					// No sorting
	    					return 0;
	    				}
	    			  
	    		  });
	    		  super.createControl(parent);
	    		  
	    	  }
	    	  
	    	 
	      };
	  
	    }
	  
	 return null;
	}
	
    protected boolean isImportant(IWorkbenchPart part) {
    	// Only Kumbang Modeller is important
        return (part instanceof KumbangModellerEditor);
    }


	public KumbangModellerEditor getEditor() {
		return editor;
	}
}
