/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.wizards;

import kumbang.modeller.ui.editor.KumbangEditorInput;
import kumbang.modeller.ui.editor.KumbangModellerEditor;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;

public class KumbangModelCreationWizard extends Wizard implements INewWizard {

	private IStructuredSelection selection;
	private KumbangOptionsCreationPage optionsPage;
	private KumbangFileCreationPage filePage;


	public boolean performFinish() {

		String fileName = filePage.getFileName();
		
		if (!fileName.endsWith(".kbm"))
			filePage.setFileName(fileName.concat(".kbm"));

		KumbangEditorInput input = new KumbangEditorInput(optionsPage.getModelName(), optionsPage.getModelType(), new FileEditorInput(filePage.createNewFile()));
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		try {
		
			IDE.openEditor(page, input, KumbangModellerEditor.EDITOR_ID, true);
		} catch (PartInitException e) {
			filePage.setErrorMessage(e.getLocalizedMessage());
			return false;
		}
		
		return true;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
	
	public void addPages() {
		filePage = new KumbangFileCreationPage(selection);
		optionsPage = new KumbangOptionsCreationPage(filePage);
		
		super.addPage(optionsPage);
		super.addPage(filePage);
	}

	
}
