/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.modeller.ui.wizards;

import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.core.model.core.KumbangModel;
import kumbang.modeller.ui.dialogs.LayoutFactory;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class KumbangOptionsCreationPage extends WizardPage {

	private Text modelNameText;
	private Button kumbang;
	private Button forfamel;
	private Button koalish;
	private KumbangFileCreationPage filePage;
//	private Button asObject;
//	private Button asText;



	 public KumbangOptionsCreationPage(KumbangFileCreationPage filePage) {
		super("New Kumbang Model");
		setTitle("Create a new Kumbang configuration model");
		setMessage("Choose the name and options for the model");
		this.filePage = filePage;
		setPageComplete(false);
	}

	public void createControl(Composite parent) {
		 Composite topLevel = new Composite(parent, SWT.NULL);
		 setControl(topLevel);
		 topLevel.setLayout(new GridLayout());
		 topLevel.setLayoutData(new GridData(GridData.FILL_BOTH));
		 
		 Composite nameComposite = new Composite(topLevel, SWT.NULL);
		 nameComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		 nameComposite.setLayout(new GridLayout(3, true));
		 
		 Label label = new Label(nameComposite, SWT.NULL);
		 label.setText("Model name:");
		 GridData labelData = new GridData();
		 label.setLayoutData(labelData);
		 
		 GridData nameData = new GridData(GridData.FILL_HORIZONTAL);
		 nameData.horizontalSpan = 3;
		 modelNameText = new Text(nameComposite, SWT.BORDER);
		 modelNameText.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		 modelNameText.setLayoutData(nameData);
		 modelNameText.addModifyListener(new ModifyListener(){

			public void modifyText(ModifyEvent e) {
				filePage.setFileName(modelNameText.getText() + ".kbm");
				if (modelNameText.getText().length() > 0)
					setPageComplete(true);
				else setPageComplete(false);
				
				
			}});
		 Group typeGroup = new Group(topLevel, SWT.NULL);;
		 typeGroup.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		 typeGroup.setLayout(new GridLayout());
		 typeGroup.setText("Select model type (can be changed later)");
		 
		 kumbang = new Button(typeGroup, SWT.RADIO);
		 kumbang.setText("Kumbang (both features and components)");
		 kumbang.setSelection(true);
		 kumbang.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		 
		 forfamel = new Button(typeGroup, SWT.RADIO);
		 forfamel.setText("Forfamel (only features)");
		 forfamel.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		 
		 koalish = new Button(typeGroup, SWT.RADIO);
		 koalish.setText("Koalish (only components)");
		 koalish.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		 /*
		 Group saveGroup = new Group(topLevel, SWT.NULL);;
		// saveGroup.setLayoutData(LayoutFactory.getInstance().getHorizontalFillGridData());
		 saveGroup.setLayout(new GridLayout());
		 saveGroup.setText("Select the save format of the model (can be changed later)");
		 
		 asObject = new Button(saveGroup, SWT.RADIO);
		 asObject.setText("Save as an object");
		 asObject.setLayoutData(LayoutFactory.getInstance().getFillGridData());
		 
		 asText = new Button(saveGroup, SWT.RADIO);
		 asText.setText("Save in text format");
		 asText.setLayoutData(LayoutFactory.getInstance().getFillGridData());
	*/
	 }

	public String getModelName() {
		return modelNameText.getText();
	}
	
	public String getModelType() {
		if (kumbang.getSelection())
			return KumbangModel.MODEL_KIND;
		if (forfamel.getSelection())
			return ForfamelModel.MODEL_KIND;
		if (koalish.getSelection())
			return KoalishModel.MODEL_KIND;
		return null;
	}

	
}
